#!/usr/bin/python3
"""Integration tests for format_films_country_data and """

import pytest

from ArWikiCats.translations import Nat_women, film_keys_for_female
from ArWikiCats.translations_formats import MultiDataFormatterDataDouble, format_films_country_data


@pytest.fixture
def yc_bot() -> MultiDataFormatterDataDouble:
    # Template data with both nationality and sport placeholders
    formatted_data = {
        "{nat_en} films": "أفلام {nat_ar}",
        "{nat_en} {film_key} films": "أفلام {film_ar} {nat_ar}",
        "{nat_en} {film_key} television commercials": "إعلانات تجارية تلفزيونية {film_ar} {nat_ar}",
    }
    other_formatted_data = {
        "{film_key} films": "أفلام {film_ar}",
        "{film_key} television commercials": "إعلانات تجارية تلفزيونية {film_ar}",
    }

    # film_keys_for_female
    data_list2 = {
        "action comedy": "حركة كوميدية",
        "action thriller": "إثارة حركة",
        "action": "حركة",
        "drama": "درامية",
        "upcoming": "قادمة",
        "horror": "رعب",
        "black-and-white": "أبيض وأسود",
        "psychological horror": "رعب نفسي",
    }

    put_label_last = {
        "low-budget",
        "christmas",
        "lgbtq-related",
        "upcoming",
    }

    bot = format_films_country_data(
        formatted_data=formatted_data,
        data_list=Nat_women,
        key_placeholder="{nat_en}",
        value_placeholder="{nat_ar}",
        data_list2=film_keys_for_female,
        key2_placeholder="{film_key}",
        value2_placeholder="{film_ar}",
        text_after="",
        text_before="",
        other_formatted_data=other_formatted_data,
    )
    bot.other_bot.update_put_label_last(put_label_last)

    return bot


test_data_standard = {
    # standard
    # yc_bot.search_all(category)
    "yemeni films": "أفلام يمنية",
    "Yemeni action films": "أفلام حركة يمنية",
    "Yemeni action drama films": "أفلام حركة درامية يمنية",
    "Yemeni upcoming horror films": "أفلام رعب قادمة يمنية",
    "Yemeni horror upcoming films": "أفلام رعب قادمة يمنية",
    "Yemeni upcoming films": "أفلام قادمة يمنية",
    # films keys
    "3d low-budget films": "أفلام ثلاثية الأبعاد منخفضة التكلفة",
    "low-budget 3d films": "أفلام ثلاثية الأبعاد منخفضة التكلفة",
    "heist historical television commercials": "إعلانات تجارية تلفزيونية سرقة تاريخية",
    "adult animated supernatural films": "أفلام رسوم متحركة خارقة للطبيعة للكبار",
    "heist holocaust films": "أفلام سرقة هولوكوستية",
    "heist hood films": "أفلام سرقة هود",
    "heist horror films": "أفلام سرقة رعب",
    "heist independent films": "أفلام سرقة مستقلة",
    "heist interactive films": "أفلام سرقة تفاعلية",
    "heist internet films": "أفلام سرقة إنترنت",
    "heist japanese horror films": "أفلام سرقة رعب يابانية",
    "heist joker films": "أفلام سرقة جوكر",
    "heist kaiju films": "أفلام سرقة كايجو",
    "heist kung fu films": "أفلام سرقة كونغ فو",
    "heist latin films": "أفلام سرقة لاتينية",
    "heist legal films": "أفلام سرقة قانونية",
    "psychological horror black-and-white films": "أفلام رعب نفسي أبيض وأسود",
    "psychological horror bollywood films": "أفلام رعب نفسي بوليوود",
    "action sports films": "أفلام حركة رياضية",
    "action spy films": "أفلام حركة تجسسية",
    "action street fighter films": "أفلام حركة قتال شوارع",
    "action student films": "أفلام حركة طلاب",
    "action submarines films": "أفلام حركة غواصات",
    "action super robot films": "أفلام حركة آلية خارقة",
    "action supernatural films": "أفلام حركة خارقة للطبيعة",
    "action supernatural drama films": "أفلام حركة دراما خارقة للطبيعة",
    "action survival films": "أفلام حركة البقاء على قيد الحياة",
    "action teen films": "أفلام حركة مراهقة",
    "action television films": "أفلام حركة تلفزيونية",
    "action thriller 3d films": "أفلام إثارة حركة ثلاثية الأبعاد",
    "action thriller 4d films": "أفلام إثارة حركة رباعية الأبعاد",
    "action thriller action films": "أفلام إثارة حركة حركة",
    "action thriller action comedy films": "أفلام إثارة حركة حركة كوميدية",
    "action thriller adaptation films": "أفلام إثارة حركة مقتبسة",
    "action thriller adult animated films": "أفلام إثارة حركة رسوم متحركة للكبار",
    "action thriller adult animated drama films": "أفلام إثارة حركة رسوم متحركة دراما للكبار",
    "action thriller adult animated supernatural films": "أفلام إثارة حركة رسوم متحركة خارقة للطبيعة للكبار",
    "action thriller adventure films": "أفلام إثارة حركة مغامرات",
    "action thriller animated films": "أفلام إثارة حركة رسوم متحركة",
    "action thriller animated science films": "أفلام إثارة حركة علمية رسوم متحركة",
    "action thriller animated short films": "أفلام إثارة حركة رسوم متحركة قصيرة",
    "psychological horror buddy films": "أفلام رعب نفسي رفقاء",
    "psychological horror cancelled films": "أفلام رعب نفسي ملغية",
}


@pytest.mark.parametrize("category,expected", test_data_standard.items(), ids=test_data_standard.keys())
def test_year_country_combinations(yc_bot: MultiDataFormatterDataDouble, category: str, expected: str) -> None:
    """
    Test
    """
    result = yc_bot.search_all(category)
    assert result == expected


country_bot_data = [
    # without films keys to test yc_bot.country_bot.search(category)
    ("Yemeni films", "أفلام يمنية"),
]


@pytest.mark.parametrize("category,expected", country_bot_data, ids=[x[0] for x in country_bot_data])
def test_country_bot(yc_bot: MultiDataFormatterDataDouble, category: str, expected: str) -> None:
    """
    Test
    """
    result = yc_bot.country_bot.search(category)
    assert result == expected


other_bot_data = {
    # without nationality to test yc_bot.other_bot.search(category)
    "black-and-white films": "أفلام أبيض وأسود",
    "black-and-white action films": "أفلام أبيض وأسود حركة",
    "drama action television commercials": "إعلانات تجارية تلفزيونية درامية حركة",
    "horror upcoming films": "أفلام رعب قادمة",
    "psychological horror films": "أفلام رعب نفسي",
    "psychological horror black-and-white films": "أفلام رعب نفسي أبيض وأسود",
}


@pytest.mark.parametrize("category,expected", other_bot_data.items(), ids=other_bot_data.keys())
def test_other_bot(yc_bot: MultiDataFormatterDataDouble, category: str, expected: str) -> None:
    """
    Test
    """
    result = yc_bot.other_bot.search(category)
    assert result == expected


# --- match_key --------------------------------------------------------

test_match_key_data = {
    "Yemeni action films": "action",
    "Yemeni action drama films": "action drama",
    "Yemeni upcoming horror films": "upcoming horror",
    "Yemeni black-and-white films": "black-and-white",
}


@pytest.mark.parametrize("category, expected", test_match_key_data.items(), ids=test_match_key_data.keys())
@pytest.mark.fast
def test_match_key(yc_bot: MultiDataFormatterDataDouble, category: str, expected: str) -> None:
    result = yc_bot.other_bot.match_key(category)
    assert result == expected



"""
Tests
"""

import pytest

from ArWikiCats.new_resolvers.jobs_resolvers.mens import mens_resolver_labels, nat_and_gender_keys

test_data2 = {
    # Category:Turkish expatriate sports-people
    "Category:Turkish expatriate sports-people": "تصنيف:رياضيون أتراك مغتربون",
    # nat
    "welsh people": "ويلزيون",
    "yemeni people": "يمنيون",
    # "abkhazian-american": "أبخازيون أمريكيون",
    # "abkhazian-american people": "أبخازيون أمريكيون",
    # jobs
    "eugenicists": "علماء متخصصون في تحسين النسل",
    "politicians who committed suicide": "سياسيون أقدموا على الانتحار",
    "writers people": "أعلام كتاب",
    "archers": "نبالون",
    "male archers": "نبالون ذكور",
    "football managers": "مدربو كرة قدم",
    # jobs + expatriate
    "expatriate football managers": "مدربو كرة قدم مغتربون",
    "expatriate male actors": "ممثلون ذكور مغتربون",
    "expatriate actors": "ممثلون مغتربون",
    "male actors": "ممثلون ذكور",
    # nat + jobs
    "yemeni eugenicists": "علماء يمنيون متخصصون في تحسين النسل",
    "yemeni politicians who committed suicide": "سياسيون يمنيون أقدموا على الانتحار",
    "yemeni contemporary artists": "فنانون يمنيون معاصرون",
    "yemeni writers": "كتاب يمنيون",
    "yemeni male writers": "كتاب ذكور يمنيون",
    "greek male writers": "كتاب ذكور يونانيون",
    # "abkhazian-american archers": "نبالون أمريكيون أبخازيون",
    "greek writers blind": "كتاب يونانيون مكفوفون",
    "writers greek blind": "كتاب يونانيون مكفوفون",
}

test_data_2 = {
    "Category:Pakistani expatriate male actors": "تصنيف:ممثلون ذكور باكستانيون مغتربون",
    "Category:expatriate male actors": "تصنيف:ممثلون ذكور مغتربون",
    "Category:Pakistani expatriate footballers": "تصنيف:لاعبو كرة قدم باكستانيون مغتربون",
    "educators": "معلمون",
    "medical doctors": "أطباء",
    "singers": "مغنون",
    "northern ireland": "",
    "republic of ireland": "",
    "republic-of ireland": "",
}


@pytest.mark.parametrize("category,expected", test_data2.items(), ids=test_data2.keys())
@pytest.mark.fast
def test_nat_pattern_multi(category: str, expected: str) -> None:
    """Test all nat translation patterns."""
    result = mens_resolver_labels(category)
    assert result == expected


@pytest.mark.parametrize("category,expected", test_data_2.items(), ids=test_data_2.keys())
@pytest.mark.fast
def test_religions_2(category: str, expected: str) -> None:
    """Test all nat translation patterns."""
    result = mens_resolver_labels(category)
    assert result == expected


@pytest.mark.fast
def test_people_key() -> None:
    result = mens_resolver_labels("people")
    assert result == ""


def test_nat_and_gender_keys():
    data = nat_and_gender_keys("{en_nat}", "emigrants", "male", "{ar_nat} مهاجرون ذكور")

    assert data == {
        "{en_nat} male emigrants": "{ar_nat} مهاجرون ذكور",
        "{en_nat} emigrants male": "{ar_nat} مهاجرون ذكور",
        "male {en_nat} emigrants": "{ar_nat} مهاجرون ذكور",
    }, print(data)



import functools
import re
from dataclasses import dataclass

from ...format_bots.relation_mapping import translation_category_relations


@functools.lru_cache(maxsize=1)
def _load_pattern() -> re.Pattern:
    """Load the regex pattern for the first line of categories."""
    # These patterns depend on dynamically generated values and are compiled at runtime
    _yy = (
        r"\d+(?:th|st|rd|nd)[−–\- ](?:millennium|century)?\s*(?:BCE*)?"
        r"|\d+(?:th|st|rd|nd)[−–\- ](?:millennium|century)?"
        r"|\d+[−–\-]\d+"
        r"|\d+s\s*(?:BCE*)?"
        r"|\d+\s*(?:BCE*)?"
    ).lower()

    _MONTHSTR3 = "(?:january|february|march|april|may|june|july|august|september|october|november|december)? *"

    _sorted_mapping = sorted(
        translation_category_relations.keys(),
        key=lambda k: (-k.count(" "), -len(k)),
    )
    _in_pattern = " |".join(map(re.escape, [n.lower() for n in _sorted_mapping]))

    _reg_line_1_match = (
        rf"(?P<monthyear>{_MONTHSTR3}(?:{_yy})|)\s*" r"(?P<in>" + _in_pattern + r"|)\s*" r"(?P<country>.*|).*"
    )
    return re.compile(_reg_line_1_match, re.I)


REGEX_SEARCH_REG_LINE_1 = _load_pattern()

# Precompiled Regex Patterns
REGEX_SUB_MILLENNIUM_CENTURY = re.compile(r"[−–\-](millennium|century)", re.I)
REGEX_SUB_CATEGORY_LOWERCASE = re.compile(r"category:", re.IGNORECASE)


@dataclass
class TypiesResult:
    year_at_first: str
    year_at_first_strip: str
    In: str
    country: str
    cat_test: str


def get_cats(category_r: str) -> tuple[str, str]:
    """Normalize category strings and return raw and lowercase variants."""
    cate = REGEX_SUB_MILLENNIUM_CENTURY.sub(r"-\g<1>", category_r)
    cate3 = REGEX_SUB_CATEGORY_LOWERCASE.sub("", cate.lower())
    return cate, cate3


def get_reg_result(category_r: str) -> TypiesResult:
    """Extract structured pieces from categories that start with a year."""
    cate, cate3 = get_cats(category_r)
    cate = REGEX_SUB_CATEGORY_LOWERCASE.sub("", cate)

    cate_gory = cate.lower()
    cat_test = cate3
    match_it = REGEX_SEARCH_REG_LINE_1.search(cate_gory)

    year_first = ""
    country = ""
    In = ""

    if match_it:
        year_first = match_it.group("monthyear")
        country = match_it.group("country")
        In = match_it.group("in")

    if year_first and cate_gory.startswith(year_first):
        cat_test = cat_test.replace(year_first.lower(), "")

    if In == cate_gory or In == cate3:
        In = ""

    if In.strip() == "by":
        country = f"by {country}"

    if not year_first:
        country = ""

    return TypiesResult(
        year_at_first=year_first,
        year_at_first_strip=year_first.strip(),
        In=In,
        country=country,
        cat_test=cat_test,
    )



from typing import Any, Dict

footballers_get_endswith: Dict[str, Dict[str, Any]] = {
    " women's footballers": {
        "lab": "لاعبات {}",
        "remove": " women's footballers",
        "example": "Category:Spanish women's footballers",
    },
    " female footballers": {
        "lab": "لاعبات {}",
        "remove": " female footballers",
        "example": "Category:Brazilian female footballers",
    },
    "c. footballers": {
        "lab": "لاعبو {}",
        "remove": " footballers",
        "example": "Category:Heartland F.C. footballers",
    },
    " footballers": {
        "lab": "لاعبو {}",
        "remove": " footballers",
        "example": "Category:German footballers",
    },
}

to_get_endswith: Dict[str, Dict[str, Any]] = {
    "squad navigational boxes": {
        "lab": "صناديق تصفح تشكيلات {}",
        "example": "Category:1996 Basketball Olympic squad navigational boxes",
    },
    "sports navigational boxes": {
        "lab": "صناديق تصفح الرياضة في {}",
        "example": "Category:Yemen sports navigational boxes",
    },
    "navigational boxes": {
        "lab": "صناديق تصفح {}",
        "example": "",
    },
    "leagues seasons": {
        "lab": "مواسم دوريات {}",
        "example": "",
    },
    "alumni": {
        "lab": "خريجو {}",
        "example": "",
    },
    "board members": {
        "lab": "أعضاء مجلس {}",
        "example": "",
    },
    "faculty": {
        "lab": "أعضاء هيئة تدريس {}",
        "example": "",
    },
    "trustees": {
        "lab": "أمناء {}",
        "example": "",
    },
    "award winners": {
        "lab": "حائزو جوائز {}",
        "example": "",
    },
    "awards winners": {
        "lab": "حائزو جوائز {}",
        "example": "",
    },
    "sidebars": {
        "lab": "أشرطة جانبية {}",
        "example": "",
    },
    "charts": {
        "lab": "مخططات {}",
        "example": "",
    },
    "commissioners": {
        "lab": "مفوضو {}",
        "example": "Category:Major Indoor Soccer League (1978–1992) commissioners",
    },
    # "commentators": { "lab": "معلقو {}", "example": "Category:Major Indoor Soccer League (1978–1992) commentators", },
    "events": {
        "lab": "أحداث {}",
        "example": "",
    },
    "tournaments": {
        "lab": "بطولات {}",
        "example": "",
    },
}

to_get_startswith: Dict[str, Dict[str, Any]] = {
    "academic staff of": {
        "lab": "أعضاء هيئة تدريس {}",
        "example": "",
    },
    "association football matches navigational boxes by teams:": {
        "lab": "صناديق تصفح مباريات كرة قدم حسب الفرق:{}",
        "example": "Category:Association football matches navigational boxes by teams:Egypt",
    },
    "21st century members of ": {
        "lab": "أعضاء {} في القرن 21",
        "example": "Category:21st-century members of the Louisiana State Legislature",
    },
    "20th century members of ": {"lab": "أعضاء {} في القرن 20", "example": ""},
    "19th century members of ": {"lab": "أعضاء {} في القرن 19", "example": ""},
    "18th century members of ": {"lab": "أعضاء {} في القرن 18", "example": ""},
    "17th century members of ": {"lab": "أعضاء {} في القرن 17", "example": ""},
    "21st century women members of ": {"lab": "عضوات {} في القرن 21", "example": ""},
    "20th century women members of ": {"lab": "عضوات {} في القرن 20", "example": ""},
    "19th century women members of ": {"lab": "عضوات {} في القرن 19", "example": ""},
    "18th century women members of ": {"lab": "عضوات {} في القرن 18", "example": ""},
    "17th century women members of ": {"lab": "عضوات {} في القرن 17", "example": ""},
    "presidents of ": {"lab": "رؤساء {}", "example": ""},
    "family of ": {"lab": "عائلة {}", "example": ""},
    "lists of ": {"lab": "قوائم {}", "example": ""},
    "children of ": {"lab": "أبناء {}", "example": ""},
    "discoveries by ": {"lab": "اكتشافات بواسطة {}", "example": ""},
    "__films about ": {"lab": "أفلام عن {}", "example": ""},
}



#!/usr/bin/python3
"""
"""

import functools
import re
from typing import Dict, Optional, Union

from ..DataModel.model_data_base import FormatDataBase
from ..formats_logger import logger


class FormatDataDoubleV2(FormatDataBase):
    """
    Handles double-key template-driven category translations.

    This class extends FormatDataBase to handle categories where two adjacent
    keys from the data_list appear together (e.g., "action drama films").
    It can match both single keys and pairs of keys, combining their Arabic
    labels in the correct order, with options to sort labels or place specific
    labels last.

    Attributes:
        formatted_data (Dict[str, str]): Template patterns mapping English patterns to Arabic templates.
        data_list (Dict[str, Union[str, Dict[str, str]]]): Key-to-Arabic-label mappings for replacements.
        key_placeholder (str): Placeholder used in formatted_data keys (default: "xoxo").
        text_after (str): Text to append after the formatted label (default: "").
        text_before (str): Text to prepend before the formatted label (default: "").
        splitter (str): Separator used in patterns for splitting keys (default: " ").
        ar_joiner (str): Joiner for combining Arabic labels (default: " ").
        sort_ar_labels (bool): Whether to sort Arabic labels alphabetically (default: False).
        keys_to_split (dict): Cache mapping combined keys to their component parts.
        put_label_last (set[str] | list[str]): Keys whose labels should appear last in combinations.
        search_multi_cache (dict): Cache for combined label lookups.
        alternation (str): Regex alternation string for keys.
        pattern (re.Pattern): Regex pattern for matching single keys.
        pattern_double (re.Pattern): Regex pattern for matching two adjacent keys.

    Example:
        >>> data_list = {
        ...     "action": {"genre_label": "أكشن"},
        ...     "drama": {"genre_label": "دراما"},
        ... }
        >>> bot = FormatDataDoubleV2(
        ...     formatted_data={"{genre} films": "أفلام {genre_label}"},
        ...     data_list=data_list,
        ...     key_placeholder="{genre}",
        ... )
        >>> bot.search("action drama films")
        'أفلام أكشن دراما'
    """

    def __init__(
        self,
        formatted_data: Dict[str, str],
        data_list: Dict[str, Union[str, Dict[str, str]]],
        key_placeholder: str = "xoxo",
        text_after: str = "",
        text_before: str = "",
        splitter: str = " ",
        ar_joiner: str = " ",
        sort_ar_labels: bool = False,
        log_multi_cache: bool = True,
    ):
        """Prepare helpers for matching and formatting template-driven labels."""
        super().__init__(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder=key_placeholder,
            text_after=text_after,
            text_before=text_before,
        )
        self.log_multi_cache = log_multi_cache
        self.sort_ar_labels = sort_ar_labels
        self.keys_to_split = {}
        self.put_label_last = {}
        self.search_multi_cache = {}
        self.splitter = splitter or " "
        self.ar_joiner = ar_joiner or " "

        self.alternation: str = self.create_alternation()
        self.pattern = self.keys_to_pattern()

        self.pattern_double = self.keys_to_pattern_double()

    def update_put_label_last(self, data: list[str] | set[str]) -> None:
        self.put_label_last = data

    def _search(self, category: str) -> str:
        """End-to-end resolution."""
        logger.debug("$$$ start _search(): ")
        logger.debug(f"++++++++ _search {self.__class__.__name__} ++++++++ ")

        if self.formatted_data_ci.get(category):
            return self.formatted_data_ci[category]

        sport_key = self.match_key(category)

        if not sport_key:
            logger.debug(f"No sport key matched for {category=}")
            return ""

        sport_label = self.get_key_label(sport_key)
        if not sport_label:
            logger.debug(f'No sport label matched for sport key: "{sport_key}"')
            return ""

        logger.debug(f"sport label: {sport_label=}")

        template_label = self.get_template(sport_key, category)
        if not template_label:
            logger.debug(f'No template label matched for sport key: "{sport_key}" and {category=}')
            return ""

        logger.debug(f"template_label: {template_label=}")

        result = self.apply_pattern_replacement(template_label, sport_label)
        logger.debug(f"[] apply_pattern_replacement: {result=}")

        logger.debug(f"++++++++ end {self.__class__.__name__} ++++++++ ")

        return result

    def keys_to_pattern_double(self) -> Optional[re.Pattern[str]]:
        """
        Build a case-insensitive regex over lowercased keys of data_list.
        """
        if not self.data_list_ci:
            return None

        if self.alternation is None:
            self.alternation = self.create_alternation()

        data_pattern = rf"(?<!\w)({self.alternation})({self.splitter})({self.alternation})(?!\w)"
        return re.compile(data_pattern, re.I)

    @functools.lru_cache(maxsize=None)
    def match_key(self, category: str) -> str:
        """Return canonical lowercased key from data_list if found; else empty."""
        if not self.pattern:
            return ""

        # Normalize the category by removing extra spaces
        normalized_category = " ".join(category.split())
        logger.debug(f">!> match_key: {normalized_category=}")

        # TODO: check this
        if self.data_list_ci.get(normalized_category.lower()):
            logger.debug(f">>!!>> match_key: found in data_list_ci {normalized_category=}")
            return normalized_category.lower()

        match = self.pattern_double.search(f" {normalized_category} ")
        if match:
            first_key = match.group(1).lower()
            splitter = match.group(2).lower()
            second_key = match.group(3).lower()
            result = f"{first_key}{splitter}{second_key}"

            logger.debug(f">!> match_key: {first_key=}, {second_key=}")
            logger.debug(f">!> match_key: {result=}")
            self.keys_to_split[result] = [first_key, second_key]
            return result

        match2 = self.pattern.search(f" {normalized_category} ")
        result = match2.group(1).lower() if match2 else ""
        logger.debug(f"==== match_key {result=}")

        return result

    def apply_pattern_replacement(self, template_label: str, sport_label: Union[str, Dict[str, str]]) -> str:
        """Replace value placeholder once template is chosen."""
        logger.debug(f"[] apply_pattern_replacement: {template_label=}, ")
        logger.debug(f"[] apply_pattern_replacement: {sport_label=}, ")

        if not isinstance(sport_label, dict):
            logger.error("===> apply_pattern_replacement: sport_label not dict..")
            return template_label

        final_label = template_label

        if isinstance(sport_label, dict):
            for key, val in sport_label.items():
                if isinstance(val, str) and val:
                    final_label = final_label.replace(f"{{{key}}}", val)

        return final_label.strip()

    @functools.lru_cache(maxsize=None)
    def create_label_from_keys(self, part1: str, part2: str):
        """
        if "upcoming" in self.put_label_last we using:
            "أفلام قادمة رعب يمنية instead of "أفلام رعب قادمة يمنية"
        """

        first_label = self.data_list_ci.get(part1)
        second_label = self.data_list_ci.get(part2)

        if not first_label or not second_label:
            logger.debug(f">>> create_label_from_keys: missing label for {part1=}, {part2=}")
            return ""

        if not isinstance(first_label, dict) or not isinstance(second_label, dict):
            logger.debug(f">>> create_label_from_keys: non-dict label for {part1=}, {part2=}")
            return ""

        keys_in_2_parts = list(first_label.keys()) + list(second_label.keys())

        logger.debug(f"!!! create_label_from_keys: found label for {part1=}, {part2=}")

        compound_data = {}

        for key in keys_in_2_parts:
            compound_data[key] = ""
            first_lab = first_label.get(key, "")
            second_lab = second_label.get(key, "")
            if first_lab and second_lab:
                label = f"{first_lab}{self.ar_joiner}{second_lab}"
                # logger.debug(f"!!! create_label_from_keys: label: {label}")

                if part1 in self.put_label_last and part2 not in self.put_label_last:
                    label = f"{second_lab}{self.ar_joiner}{first_lab}"

                if self.sort_ar_labels:
                    labels_sorted = sorted([first_lab, second_lab])
                    label = self.ar_joiner.join(labels_sorted)
                compound_data[key] = label
        if self.log_multi_cache:
            self.search_multi_cache[f"{part2} {part1}"] = compound_data

        return compound_data

    def get_key_label(self, sport_key: str) -> dict[str, str]:
        """
        Return the Arabic label mapped to the provided key if present.
        """
        logger.debug(f"@@ get_key_label: {sport_key=}")

        result = self.data_list_ci.get(sport_key)
        if result:
            return result

        if self.search_multi_cache.get(sport_key.lower()):
            logger.debug(f"@@ get_key_label: found in search_multi_cache {sport_key=}")
            return self.search_multi_cache[sport_key.lower()]

        if sport_key in self.keys_to_split:
            part1, part2 = self.keys_to_split[sport_key]
            logger.debug(f"@@ get_key_label: found in keys_to_split {sport_key=}")
            return self.create_label_from_keys(part1, part2)

        return ""

    def replace_value_placeholder(self, label: str, value: Union[str, Dict[str, str]]) -> str:
        """ """
        logger.debug(f"@@ replace_value_placeholder: {label=}, {value=}")

        if not isinstance(value, dict):
            return label

        final_label = label
        for key, val in value.items():
            if isinstance(val, str) and val:
                final_label = final_label.replace(f"{{{key}}}", val)

        return final_label



import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats import resolve_arabic_category_label

data1 = {
    "Category:Irish association football managers": "تصنيف:مدربو كرة قدم أيرلنديون",
    "Category:Lists of association football players by national team": "تصنيف:قوائم لاعبو كرة قدم حسب المنتخب الوطني",
    "Category:Male long-distance runners": "تصنيف:عداؤو مسافات طويلة ذكور",
    "Category:Male runners by nationality": "تصنيف:عداؤون ذكور حسب الجنسية",
    "Category:Male steeplechase runners": "تصنيف:عداؤو موانع ذكور",
    "Category:Moroccan competitors by sports event": "تصنيف:منافسون مغاربة حسب الحدث الرياضي",
    "Category:Moroccan male middle-distance runners": "تصنيف:عداؤو مسافات متوسطة ذكور مغاربة",
    "Category:Norwegian figure skaters": "تصنيف:متزلجون فنيون نرويجيون",
    "Category:Norwegian male pair skaters": "تصنيف:متزلجون فنيون على الجليد ذكور نرويجيون",
    "Category:Norwegian male single skaters": "تصنيف:متزلجون فرديون ذكور نرويجيون",
    "Category:Water polo at the Summer Universiade": "تصنيف:كرة الماء في الألعاب الجامعية الصيفية",
    "Category:World Judo Championships": "تصنيف:بطولة العالم للجودو",
    "Category:Youth athletics competitions": "تصنيف:منافسات ألعاب قوى شبابية",
    "Category:Youth athletics": "تصنيف:ألعاب القوى للشباب",
    "Category:Youth sports competitions": "تصنيف:منافسات رياضية شبابية",
    "Category:football in 2050–51": "تصنيف:كرة القدم في 2050–51",
    "Category:nations at the universiade": "تصنيف:بلدان في الألعاب الجامعية",
    "Category:ugandan football": "تصنيف:كرة القدم الأوغندية",
    "Category:Spanish sports broadcasters": "تصنيف:مذيعون رياضيون إسبان",
    "Category:Sports broadcasters by nationality": "تصنيف:مذيعون رياضيون حسب الجنسية",
    "Category:Afghanistan national football team managers": "تصنيف:مدربو منتخب أفغانستان لكرة القدم",
    "Category:African women's national association football teams": "تصنيف:منتخبات كرة قدم وطنية إفريقية للسيدات",
    "Category:Argentina women's international footballers": "تصنيف:لاعبات منتخب الأرجنتين لكرة القدم للسيدات",
    "Category:Belgian athletics coaches": "تصنيف:مدربو ألعاب قوى بلجيكيون",
    "Category:Coaches of national cricket teams": "تصنيف:مدربو منتخبات كريكت وطنية",
    "Category:International women's basketball competitions hosted by Cuba": "تصنيف:منافسات كرة سلة دولية للسيدات استضافتها كوبا",
    "Category:Sports coaches by nationality": "تصنيف:مدربو رياضة حسب الجنسية",
    "Category:Transport companies established in 1909": "تصنيف:شركات نقل أسست في 1909",
}

data2 = {
    "Category:Female association football managers": "تصنيف:مدربات كرة قدم",
    # "Category:Coaches of the West Indies national cricket team": "",
    # "Category:Nauru international soccer players": "",
    "Category:Australia international soccer players": "تصنيف:لاعبو منتخب أستراليا لكرة القدم",
    "Category:Canada men's international soccer players": "تصنيف:لاعبو كرة قدم دوليون من كندا",
    "Category:Afghanistan women's national football team coaches": "تصنيف:مدربو منتخب أفغانستان لكرة القدم للسيدات",
    "Category:Coaches of Yemen national cricket team": "تصنيف:مدربو منتخب اليمن للكريكت",
    "Category:Cuba women's national basketball team": "تصنيف:منتخب كوبا لكرة السلة للسيدات",
    "Category:Equatorial Guinea women's national football team": "تصنيف:منتخب غينيا الاستوائية لكرة القدم للسيدات",
    "Category:Norwegian pair skaters": "تصنيف:متزلجون فنيون على الجليد نرويجيون",
    "Category:Norwegian short track speed skaters": "تصنيف:متزلجون على مسار قصير نرويجيون",
    "Category:Olympic competitors for Cape Verde": "تصنيف:منافسون أولمبيون من الرأس الأخضر",
    "Category:Olympic figure skating": "تصنيف:تزلج فني أولمبي",
    "Category:Olympic medalists in alpine skiing": "تصنيف:فائزون بميداليات أولمبية في التزلج على المنحدرات الثلجية",
    "Category:Rail transport in the United Kingdom": "تصنيف:السكك الحديدية في المملكة المتحدة",
    "Category:Republic of Ireland football managers": "تصنيف:مدربو كرة قدم أيرلنديون",
    "Category:Seasons in Omani football": "تصنيف:مواسم في كرة القدم العمانية",
    "Category:Ski jumping at the Winter Universiade": "تصنيف:القفز التزلجي في الألعاب الجامعية الشتوية",
    "Category:Skiing coaches": "تصنيف:مدربو تزلج",
    "Category:Sports competitors by nationality and competition": "تصنيف:منافسون رياضيون حسب الجنسية والمنافسة",
    "Category:Sports organisations of Andorra": "تصنيف:منظمات رياضية في أندورا",
    "Category:sports-people from Boston": "تصنيف:رياضيون من بوسطن",
    "Category:Table tennis clubs": "تصنيف:أندية كرة طاولة",
    "Category:Transport disasters in 2017": "تصنيف:كوارث نقل في 2017",
    "Category:Turkish expatriate sports-people": "تصنيف:رياضيون أتراك مغتربون",
    "Category:Universiade medalists by sport": "تصنيف:فائزون بميداليات الألعاب الجامعية حسب الرياضة",
    "Category:Universiade medalists in water polo": "تصنيف:فائزون بميداليات الألعاب الجامعية في كرة الماء",
    "Category:Association football players by under-20 national team": "تصنيف:لاعبو كرة قدم حسب المنتخب الوطني تحت 20 سنة",
    "Category:Association football players by under-21 national team": "تصنيف:لاعبو كرة قدم حسب المنتخب الوطني تحت 21 سنة",
    "Category:Association football players by under-23 national team": "تصنيف:لاعبو كرة قدم حسب المنتخب الوطني تحت 23 سنة",
    "Category:Association football players by youth national team": "تصنيف:لاعبو كرة قدم حسب المنتخب الوطني للشباب",
    "Category:Association football": "تصنيف:كرة القدم",
}
data3 = {
    "Category:Female short track speed skaters": "تصنيف:متزلجات على مسار قصير",
    "Category:Female speed skaters": "تصنيف:متزلجات سرعة",
    "Category:Figure skaters by competition": "تصنيف:متزلجون فنيون حسب المنافسة",
    "Category:Figure skating coaches": "تصنيف:مدربو تزلج فني",
    "Category:Figure skating people": "تصنيف:أعلام تزلج فني",
    "Category:Icelandic male athletes": "تصنيف:لاعبو قوى ذكور آيسلنديون",
    "Category:Icelandic male runners": "تصنيف:عداؤون ذكور آيسلنديون",
    "Category:Icelandic male steeplechase runners": "تصنيف:عداؤو موانع ذكور آيسلنديون",
    "Category:IndyCar": "تصنيف:أندي كار",
    "Category:International sports competitions hosted by Mexico": "تصنيف:منافسات رياضية دولية استضافتها المكسيك",
    "Category:Egyptian male sport shooters": "تصنيف:لاعبو رماية ذكور مصريون",
    "Category:Egyptian sport shooters": "تصنيف:لاعبو رماية مصريون",
    "Category:Emirati football in 2017": "تصنيف:كرة القدم الإماراتية في 2017",
    "Category:Emirati football in 2017–18": "تصنيف:كرة القدم الإماراتية في 2017–18",
    "Category:England amateur international footballers": "تصنيف:لاعبو منتخب إنجلترا لكرة القدم للهواة",
    "Category:Equatoguinean women's footballers": "تصنيف:لاعبات كرة قدم غينيات استوائيات",
    "Category:European national under-21 association football teams": "تصنيف:منتخبات كرة قدم وطنية أوروبية تحت 21 سنة",
    "Category:Expatriate women's association football players": "تصنيف:لاعبات كرة قدم مغتربات",
    "Category:Expatriate women's footballers by location": "تصنيف:لاعبات كرة قدم مغتربات حسب الموقع",
    "Category:Australia at the Summer Universiade": "تصنيف:أستراليا في الألعاب الجامعية الصيفية",
    "Category:Australian male sprinters": "تصنيف:عداؤون سريعون ذكور أستراليون",
    "Category:Canadian sports businesspeople": "تصنيف:شخصيات أعمال رياضيون كنديون",
    "Category:Cape Verde at the Paralympics": "تصنيف:الرأس الأخضر في الألعاب البارالمبية",
    "Category:Cape Verdean football managers": "تصنيف:مدربو كرة قدم أخضريون",
    "Category:Egyptian female sport shooters": "تصنيف:لاعبات رماية مصريات",
    "Category:Afghan competitors by sports event": "تصنيف:منافسون أفغان حسب الحدث الرياضي",
    "Category:American basketball players by ethnic or national origin": "تصنيف:لاعبو كرة سلة أمريكيون حسب الأصل العرقي أو الوطني",
    "Category:Argentina at the Universiade": "تصنيف:الأرجنتين في الألعاب الجامعية",
    "Category:Argentina at the Winter Olympics": "تصنيف:الأرجنتين في الألعاب الأولمبية الشتوية",
    "Category:Association football players by amateur national team": "تصنيف:لاعبو كرة قدم حسب المنتخب الوطني للهواة",
}

to_test = [
    ("test_sports_1", data1),
    ("test_sports_2", data2),
    ("test_sports_3", data3),
]


@pytest.mark.parametrize("category, expected", data1.items(), ids=data1.keys())
@pytest.mark.fast
def test_sports_1(category: str, expected: str) -> None:
    label = resolve_arabic_category_label(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", data2.items(), ids=data2.keys())
@pytest.mark.fast
def test_sports_2(category: str, expected: str) -> None:
    label = resolve_arabic_category_label(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", data3.items(), ids=data3.keys())
@pytest.mark.fast
def test_sports_3(category: str, expected: str) -> None:
    label = resolve_arabic_category_label(category)
    assert label == expected


@pytest.mark.parametrize("name,data", to_test)
@pytest.mark.dump
def test_sports(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_arabic_category_label)

    dump_diff(diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



#!/usr/bin/python3
"""
Comprehensive tests for FormatDataDoubleV2 class.

This module provides exhaustive tests for:
1. ar_joiner parameter (default, custom, None, empty)
2. sort_ar_labels parameter (True/False)
3. splitter parameter (default, custom, regex patterns)
4. Combination of parameters
5. Edge cases and error handling
"""

import pytest

from ArWikiCats.translations_formats import FormatDataDoubleV2


@pytest.fixture
def base_data_v2():
    """Base data fixture with dict-based labels."""
    formatted_data = {
        "{film_key} films": "أفلام {film_label}",
        "{film_key} movies": "أفلام {film_label}",
    }
    data_list = {
        "action": {"film_label": "أكشن"},
        "drama": {"film_label": "دراما"},
        "comedy": {"film_label": "كوميدي"},
        "horror": {"film_label": "رعب"},
    }
    return formatted_data, data_list


@pytest.fixture
def multi_key_data():
    """Data with multiple keys per entry."""
    formatted_data = {
        "{genre} films": "أفلام {genre_label}",
    }
    data_list = {
        "action": {"genre_label": "أكشن"},
        "drama": {"genre_label": "دراما"},
        "horror": {"genre_label": "رعب"},
    }
    return formatted_data, data_list


# ============================================================================
# Tests for ar_joiner parameter
# ============================================================================


def test_ar_joiner_default_space(base_data_v2):
    """Test default ar_joiner (space)."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
    )
    result = bot.search("action drama films")
    assert result == "أفلام أكشن دراما"


def test_ar_joiner_with_wa(base_data_v2):
    """Test ar_joiner with Arabic 'و' (and)."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        ar_joiner=" و ",
    )
    result = bot.search("action drama films")
    assert result == "أفلام أكشن و دراما"


def test_ar_joiner_with_dash(base_data_v2):
    """Test ar_joiner with dash."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        ar_joiner="-",
    )
    result = bot.search("action drama films")
    assert result == "أفلام أكشن-دراما"


def test_ar_joiner_none_defaults_to_space(base_data_v2):
    """Test ar_joiner=None defaults to space."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        ar_joiner=None,
    )
    result = bot.search("action drama films")
    assert result == "أفلام أكشن دراما"


def test_ar_joiner_empty_defaults_to_space(base_data_v2):
    """Test ar_joiner='' defaults to space."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        ar_joiner="",
    )
    result = bot.search("action drama films")
    assert result == "أفلام أكشن دراما"


def test_ar_joiner_with_comma(base_data_v2):
    """Test ar_joiner with comma and space."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        ar_joiner="، ",
    )
    result = bot.search("action drama films")
    assert result == "أفلام أكشن، دراما"


# ============================================================================
# Tests for sort_ar_labels parameter
# ============================================================================


def test_sort_ar_labels_false_preserves_order(base_data_v2):
    """Test sort_ar_labels=False preserves original order."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        sort_ar_labels=False,
        log_multi_cache=False,
    )
    result1 = bot.search("action drama films")
    result2 = bot.search("drama action films")
    assert result1 == "أفلام أكشن دراما"
    assert result2 == "أفلام دراما أكشن"


def test_sort_ar_labels_true_sorts_alphabetically(base_data_v2):
    """Test sort_ar_labels=True sorts labels alphabetically."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        sort_ar_labels=True,
    )
    # أكشن < دراما alphabetically
    result1 = bot.search("action drama films")
    result2 = bot.search("drama action films")
    assert result1 == "أفلام أكشن دراما"
    assert result2 == "أفلام أكشن دراما"


def test_sort_ar_labels_with_comedy_drama(base_data_v2):
    """Test sort_ar_labels with comedy and drama."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        sort_ar_labels=True,
    )
    # دراما < كوميدي alphabetically
    result = bot.search("comedy drama films")
    assert result == "أفلام دراما كوميدي"


def test_sort_ar_labels_with_horror_action(base_data_v2):
    """Test sort_ar_labels with horror and action."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        sort_ar_labels=True,
    )
    # أكشن < رعب alphabetically
    result1 = bot.search("horror action films")
    result2 = bot.search("action horror films")
    assert result1 == "أفلام أكشن رعب"
    assert result2 == "أفلام أكشن رعب"


# ============================================================================
# Tests for splitter parameter
# ============================================================================


def test_splitter_default_space(base_data_v2):
    """Test default splitter (space)."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
    )
    result = bot.search("action drama films")
    assert result == "أفلام أكشن دراما"


def test_splitter_underscore(base_data_v2):
    """Test splitter with underscore pattern."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        splitter="[_ ]",
    )
    # Should match both underscore and space
    result1 = bot.search("action_drama films")
    result2 = bot.search("action drama films")
    assert result1 == "أفلام أكشن دراما"
    assert result2 == "أفلام أكشن دراما"


def test_splitter_dash(base_data_v2):
    """Test splitter with dash."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        splitter="-",
    )
    result = bot.search("action-drama films")
    assert result == "أفلام أكشن دراما"


def test_splitter_regex_dot(base_data_v2):
    """Test splitter with regex dot (matches any character)."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        splitter=".",
    )
    # Dot matches any character
    result1 = bot.search("action.drama films")
    result2 = bot.search("action_drama films")
    result3 = bot.search("action drama films")
    assert result1 == "أفلام أكشن دراما"
    assert result2 == "أفلام أكشن دراما"
    assert result3 == "أفلام أكشن دراما"


def test_splitter_multiple_chars(base_data_v2):
    """Test splitter with multiple character pattern."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        splitter="[_\\- ]",
    )
    # Should match underscore, dash, or space
    result1 = bot.search("action_drama films")
    result2 = bot.search("action-drama films")
    result3 = bot.search("action drama films")
    assert result1 == "أفلام أكشن دراما"
    assert result2 == "أفلام أكشن دراما"
    assert result3 == "أفلام أكشن دراما"


# ============================================================================
# Tests for combined parameters
# ============================================================================


def test_ar_joiner_and_sort_combined(base_data_v2):
    """Test ar_joiner with sort_ar_labels."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        ar_joiner=" و ",
        sort_ar_labels=True,
    )
    result1 = bot.search("action drama films")
    result2 = bot.search("drama action films")
    assert result1 == "أفلام أكشن و دراما"
    assert result2 == "أفلام أكشن و دراما"


def test_splitter_and_ar_joiner_combined(base_data_v2):
    """Test splitter with ar_joiner."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        splitter="-",
        ar_joiner=" و ",
    )
    result = bot.search("action-drama films")
    assert result == "أفلام أكشن و دراما"


def test_all_three_parameters_combined(base_data_v2):
    """Test splitter, ar_joiner, and sort_ar_labels together."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        splitter="[_\\- ]",
        ar_joiner="، ",
        sort_ar_labels=True,
    )
    result1 = bot.search("drama_action films")
    result2 = bot.search("action-drama movies")
    assert result1 == "أفلام أكشن، دراما"
    assert result2 == "أفلام أكشن، دراما"


# ============================================================================
# Tests for multi-key data
# ============================================================================


def test_multi_key_default(multi_key_data):
    """Test multi-key data with default settings."""
    formatted_data, data_list = multi_key_data
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{genre}",
    )
    result = bot.search("action drama films")
    assert result == "أفلام أكشن دراما"


def test_multi_key_with_ar_joiner(multi_key_data):
    """Test multi-key data with custom ar_joiner."""
    formatted_data, data_list = multi_key_data
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{genre}",
        ar_joiner=" و ",
    )
    result = bot.search("action drama films")
    assert result == "أفلام أكشن و دراما"


def test_multi_key_with_sort(multi_key_data):
    """Test multi-key data with sort_ar_labels."""
    formatted_data, data_list = multi_key_data
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{genre}",
        sort_ar_labels=True,
    )
    result1 = bot.search("action drama films")
    result2 = bot.search("drama action films")
    # Both should be sorted: أكشن < دراما
    assert result1 == "أفلام أكشن دراما"
    assert result2 == "أفلام أكشن دراما"


# ============================================================================
# Tests for single key matching
# ============================================================================


def test_single_key_match(base_data_v2):
    """Test matching single key (not double)."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
    )
    result = bot.search("action films")
    assert result == "أفلام أكشن"


def test_single_key_with_different_template(base_data_v2):
    """Test single key with different template."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
    )
    result = bot.search("comedy movies")
    assert result == "أفلام كوميدي"


# ============================================================================
# Tests for edge cases
# ============================================================================


def test_no_match_returns_empty(base_data_v2):
    """Test that non-matching category returns empty string."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
    )
    result = bot.search("science fiction films")
    assert result == ""


def test_partial_match_returns_empty(base_data_v2):
    """Test that partial match returns empty string."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
    )
    result = bot.search("action science films")
    assert result == ""


def test_case_insensitive_matching(base_data_v2):
    """Test case-insensitive matching."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
    )
    result1 = bot.search("ACTION DRAMA films")
    result2 = bot.search("Action Drama Films")
    result3 = bot.search("action drama films")
    assert result1 == "أفلام أكشن دراما"
    assert result2 == "أفلام أكشن دراما"
    assert result3 == "أفلام أكشن دراما"


def test_extra_spaces_normalized(base_data_v2):
    """Test that extra spaces are normalized."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
    )
    result = bot.search("action  drama   films")
    assert result == "أفلام أكشن دراما"


# ============================================================================
# Tests for put_label_last functionality
# ============================================================================


def test_put_label_last_basic(base_data_v2):
    """Test put_label_last functionality with log_multi_cache=False."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        log_multi_cache=False,
    )
    bot.update_put_label_last(["action"])
    # When "action" is in put_label_last, it should appear last
    result = bot.search("action drama films")
    assert result == "أفلام دراما أكشن"


def test_put_label_last_with_reverse_order(base_data_v2):
    """Test put_label_last with reverse order input."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        log_multi_cache=False,
    )
    bot.update_put_label_last(["drama"])
    result = bot.search("action drama films")
    # drama should be last, so: أكشن دراما
    assert result == "أفلام أكشن دراما"


def test_put_label_last_with_ar_joiner(base_data_v2):
    """Test put_label_last with custom ar_joiner."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        ar_joiner=" و ",
        log_multi_cache=False,
    )
    bot.update_put_label_last(["action"])
    result = bot.search("action drama films")
    assert result == "أفلام دراما و أكشن"


def test_put_label_last_both_in_list(base_data_v2):
    """Test when both keys are in put_label_last."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        log_multi_cache=False,
    )
    bot.update_put_label_last(["action", "drama"])
    # When both are in the list, order should be preserved
    result = bot.search("action drama films")
    assert result == "أفلام أكشن دراما"


# ============================================================================
# Tests for log_multi_cache parameter
# ============================================================================


def test_log_multi_cache_true_default(base_data_v2):
    """Test log_multi_cache=True (default) caches results."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        log_multi_cache=True,
    )
    result = bot.search("action drama films")
    assert result == "أفلام أكشن دراما"
    # Check that cache was populated
    assert len(bot.search_multi_cache) > 0


def test_log_multi_cache_false_no_caching(base_data_v2):
    """Test log_multi_cache=False doesn't cache results."""
    formatted_data, data_list = base_data_v2
    bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        log_multi_cache=False,
    )
    result = bot.search("action drama films")
    assert result == "أفلام أكشن دراما"
    # Check that cache was NOT populated
    assert len(bot.search_multi_cache) == 0


def test_log_multi_cache_affects_put_label_last(base_data_v2):
    """Test that log_multi_cache=False allows put_label_last to work correctly."""
    formatted_data, data_list = base_data_v2
    # With caching enabled, put_label_last might not work after first call
    bot_cached = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        log_multi_cache=True,
    )
    bot_cached.update_put_label_last(["action"])
    result_cached = bot_cached.search("action drama films")

    # With caching disabled, put_label_last works correctly
    bot_no_cache = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        log_multi_cache=False,
    )
    bot_no_cache.update_put_label_last(["action"])
    result_no_cache = bot_no_cache.search("action drama films")

    # The no-cache version should respect put_label_last
    assert result_no_cache == "أفلام دراما أكشن"



"""
Tests
"""

import pytest

from ArWikiCats import resolve_label_ar

data_fast = {
    "4th senate of spain": "مجلس شيوخ إسبانيا",
    "19th government of turkey": "حكومة تركيا",
    "11th government of turkey": "حكومة تركيا",
    "1330 in men's international football": "كرة قدم دولية رجالية في 1340",
}


@pytest.mark.parametrize("category, not_expected", data_fast.items(), ids=data_fast.keys())
@pytest.mark.fast
def test_get_country2_one(category: str, not_expected: str) -> None:
    label = resolve_label_ar(category)
    assert label != not_expected



#
import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats import resolve_label_ar

data_2 = {
    "Fictional Buddhist nuns": "راهبات بوذيات خياليات",
    "20th-century American Buddhist nuns": "راهبات بوذيات أمريكيات في القرن 20",
    "21st-century American Buddhist nuns": "راهبات بوذيات أمريكيات في القرن 21",
    "10th-century Buddhist nuns": "راهبات بوذيات في القرن 10",
    "10th-century Christian nuns": "راهبات مسيحيات في القرن 10",
    "11th-century Buddhist nuns": "راهبات بوذيات في القرن 11",
    "11th-century Christian nuns": "راهبات مسيحيات في القرن 11",
    "12th-century Buddhist nuns": "راهبات بوذيات في القرن 12",
    "12th-century Christian nuns": "راهبات مسيحيات في القرن 12",
    "13th-century Buddhist nuns": "راهبات بوذيات في القرن 13",
    "13th-century Christian nuns": "راهبات مسيحيات في القرن 13",
    "14th-century Buddhist nuns": "راهبات بوذيات في القرن 14",
    "14th-century Christian nuns": "راهبات مسيحيات في القرن 14",
    "15th-century Buddhist nuns": "راهبات بوذيات في القرن 15",
    "15th-century Christian nuns": "راهبات مسيحيات في القرن 15",
    "16th-century Buddhist nuns": "راهبات بوذيات في القرن 16",
    "16th-century Christian nuns": "راهبات مسيحيات في القرن 16",
    "17th-century Buddhist nuns": "راهبات بوذيات في القرن 17",
    "17th-century Christian nuns": "راهبات مسيحيات في القرن 17",
    "18th-century Buddhist nuns": "راهبات بوذيات في القرن 18",
    "18th-century Christian nuns": "راهبات مسيحيات في القرن 18",
    "19th-century Anglican nuns": "راهبات أنجليكيات في القرن 19",
    "19th-century Australian Christian nuns": "راهبات مسيحيات أستراليات في القرن 19",
    "19th-century British Anglican nuns": "راهبات أنجليكيات بريطانيات في القرن 19",
    "19th-century Buddhist nuns": "راهبات بوذيات في القرن 19",
    "19th-century Christian nuns": "راهبات مسيحيات في القرن 19",
    "1st-century Buddhist nuns": "راهبات بوذيات في القرن 1",
    "20th-century Anglican nuns": "راهبات أنجليكيات في القرن 20",
    "20th-century Australian Christian nuns": "راهبات مسيحيات أستراليات في القرن 20",
    "20th-century British Anglican nuns": "راهبات أنجليكيات بريطانيات في القرن 20",
    "20th-century Buddhist nuns": "راهبات بوذيات في القرن 20",
    "20th-century Christian nuns": "راهبات مسيحيات في القرن 20",
    "21st-century Anglican nuns": "راهبات أنجليكيات في القرن 21",
    "21st-century Australian Christian nuns": "راهبات مسيحيات أستراليات في القرن 21",
    "21st-century British Anglican nuns": "راهبات أنجليكيات بريطانيات في القرن 21",
    "21st-century Buddhist nuns": "راهبات بوذيات في القرن 21",
    "21st-century Christian nuns": "راهبات مسيحيات في القرن 21",
    "3rd-century BC Buddhist nuns": "راهبات بوذيات في القرن 3 ق م",
    "4th-century Buddhist nuns": "راهبات بوذيات في القرن 4",
    "4th-century Christian nuns": "راهبات مسيحيات في القرن 4",
    "5th-century Buddhist nuns": "راهبات بوذيات في القرن 5",
    "5th-century Christian nuns": "راهبات مسيحيات في القرن 5",
    "6th-century Buddhist nuns": "راهبات بوذيات في القرن 6",
    "6th-century Christian nuns": "راهبات مسيحيات في القرن 6",
    "7th-century Buddhist nuns": "راهبات بوذيات في القرن 7",
    "7th-century Christian nuns": "راهبات مسيحيات في القرن 7",
    "8th-century Buddhist nuns": "راهبات بوذيات في القرن 8",
    "8th-century Christian nuns": "راهبات مسيحيات في القرن 8",
    "9th-century Buddhist nuns": "راهبات بوذيات في القرن 9",
    "9th-century Christian nuns": "راهبات مسيحيات في القرن 9",
    "American Buddhist nuns": "راهبات بوذيات أمريكيات",
    "American Christian nuns": "راهبات مسيحيات أمريكيات",
    "American Hindu nuns": "راهبات هندوسيات أمريكيات",
    "Australian Christian nuns": "راهبات مسيحيات أستراليات",
    "Belgian Buddhist nuns": "راهبات بوذيات بلجيكيات",
    "British Anglican nuns": "راهبات أنجليكيات بريطانيات",
    "British Buddhist nuns": "راهبات بوذيات بريطانيات",
    "Buddhist nuns by century": "راهبات بوذيات حسب القرن",
    "Buddhist nuns by nationality": "راهبات بوذيات حسب الجنسية",
    "Buddhist nuns of Nara period": "راهبات بوذيات في فترة نارا",
    "Chinese Buddhist nuns": "راهبات بوذيات صينيات",
    "Christian nuns by century": "راهبات مسيحيات حسب القرن",
    "Fictional Christian nuns": "راهبات مسيحيات خياليات",
    "French Buddhist nuns": "راهبات بوذيات فرنسيات",
    "German Buddhist nuns": "راهبات بوذيات ألمانيات",
    "Indian Buddhist nuns": "راهبات بوذيات هنديات",
    "Indian Hindu nuns": "راهبات هندوسيات هنديات",
    "Indonesian Buddhist nuns": "راهبات بوذيات إندونيسيات",
    "Irish Buddhist nuns": "راهبات بوذيات أيرلنديات",
    "Italian Buddhist nuns": "راهبات بوذيات إيطاليات",
    "Japanese Buddhist nuns by period": "راهبات بوذيات يابانيات حسب الحقبة",
    "Japanese Buddhist nuns": "راهبات بوذيات يابانيات",
    "Korean Buddhist nuns": "راهبات بوذيات كوريات",
    "Nepalese Buddhist nuns": "راهبات بوذيات نيباليات",
    "Scottish Buddhist nuns": "راهبات بوذيات إسكتلنديات",
    "Singaporean Buddhist nuns": "راهبات بوذيات سنغافوريات",
    "South Korean Buddhist nuns": "راهبات بوذيات كوريات جنوبيات",
    "Taiwanese Buddhist nuns": "راهبات بوذيات تايوانيات",
    "Thai Buddhist nuns": "راهبات بوذيات تايلنديات",
    "Vietnamese Buddhist nuns": "راهبات بوذيات فيتناميات",
}

to_test = [
    ("test_nuns_2", data_2),
]


@pytest.mark.parametrize("category, expected", data_2.items(), ids=data_2.keys())
@pytest.mark.fast
def test_nuns_2(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


@pytest.mark.parametrize("name,data", to_test)
@pytest.mark.dump
def test_peoples(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_label_ar)

    dump_diff(diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



"""
Integration tests for separator_lists_fixing and add_in_tab functions.

These tests verify the functions work correctly with real-world scenarios
and actual data dependencies.
"""

import pytest

from ArWikiCats.legacy_bots.ma_bots2.ar_lab_bot import (
    add_in_tab,
    separator_lists_fixing,
)


class TestSeparatorListsFixingIntegration:
    """Integration tests for separator_lists_fixing with real data."""

    @pytest.mark.fast
    def test_military_installations_in(self) -> None:
        """Test real-world example: military installations in."""
        result = separator_lists_fixing(
            type_label="منشآت عسكرية", separator_stripped="in", type_lower="military installations in"
        )
        assert result == "منشآت عسكرية في"
        assert " في" in result
        assert result.endswith("في")

    @pytest.mark.fast
    def test_sport_at_location(self) -> None:
        """Test real-world example: sport at location."""
        result = separator_lists_fixing(type_label="رياضة", separator_stripped="at", type_lower="sport at")
        assert result == "رياضة في"

    @pytest.mark.fast
    def test_complex_label_with_in(self) -> None:
        """Test complex Arabic label with 'in' separator."""
        result = separator_lists_fixing(
            type_label="أحداث رياضية", separator_stripped="in", type_lower="sports events in"
        )
        assert result == "أحداث رياضية في"

    @pytest.mark.fast
    def test_preserves_existing_في(self) -> None:
        """Test that existing 'في' is preserved."""
        result = separator_lists_fixing(
            type_label="أحداث رياضية في", separator_stripped="in", type_lower="sports events in"
        )
        # Should not add duplicate في
        assert result.count(" في") == 1

    @pytest.mark.fast
    def test_multiple_word_labels(self) -> None:
        """Test with multi-word Arabic labels."""
        result = separator_lists_fixing(
            type_label="المنشآت الرياضية الكبرى", separator_stripped="in", type_lower="major sports facilities in"
        )
        assert result == "المنشآت الرياضية الكبرى في"


class TestAddInTabIntegration:
    """Integration tests for add_in_tab with real data."""

    @pytest.mark.fast
    def test_athletes_from_country(self) -> None:
        """Test real-world example: athletes from country."""
        result = add_in_tab(type_label="رياضيون", type_lower="athletes", separator_stripped="from")
        assert result == "رياضيون من "
        assert " من " in result
        assert result.endswith("من ")

    @pytest.mark.fast
    def test_preserves_existing_من(self) -> None:
        """Test that existing 'من' is preserved."""
        result = add_in_tab(type_label="رياضيون من", type_lower="athletes", separator_stripped="from")
        # Should not add duplicate من
        assert result.count(" من") == 1

    @pytest.mark.fast
    @pytest.mark.slow
    def test_footballers_of_team(self) -> None:
        """Test real-world example: footballers of team."""
        # This test requires actual data dependencies
        result = add_in_tab(type_label="لاعبو كرة قدم", type_lower="footballers of", separator_stripped="in")
        # Result depends on get_pop_All_18 and check_key_new_players
        assert isinstance(result, str)
        # Should either add 'من' or return unchanged
        assert result.startswith("لاعبو كرة قدم")

    @pytest.mark.fast
    def test_with_real_arabic_labels(self) -> None:
        """Test with various real Arabic category labels."""
        test_cases = [
            ("رياضيون", "athletes", "from", "رياضيون من "),
            ("لاعبون", "players", "from", "لاعبون من "),
            ("مدربون", "coaches", "from", "مدربون من "),
        ]

        for type_label, type_lower, separator, expected in test_cases:
            result = add_in_tab(type_label, type_lower, separator)
            assert result == expected, f"Failed for {type_label}"


class TestCombinedUsage:
    """Tests for using both functions together as they would be in practice."""

    @pytest.mark.fast
    def test_from_then_in_separator(self) -> None:
        """Test from separator followed by checking in conditions."""
        label = "رياضيون"

        # Apply from separator
        label = add_in_tab(label, "athletes", "from")
        assert label == "رياضيون من "

        # في should not be added if we use separator_lists_fixing after
        # because it doesn't apply to 'from' separator
        label_with_from = separator_lists_fixing(label, "from", "athletes")
        assert label_with_from == label  # unchanged

    @pytest.mark.fast
    def test_workflow_for_category_processing(self) -> None:
        """Test a realistic workflow for processing a category."""
        # Simulate processing "Military installations in France"
        type_label = "منشآت عسكرية"
        type_lower = "military installations in"
        separator = "in"

        # Step 1: Fix separator
        type_label = separator_lists_fixing(type_label, separator, type_lower)
        assert type_label == "منشآت عسكرية في"

        # Step 2: Check if we need to add من
        type_label = add_in_tab(type_label, type_lower, separator)
        # Should not add من because في is already there
        assert " من" not in type_label


class TestRealWorldScenarios:
    """Tests based on actual Wikipedia category patterns."""

    @pytest.mark.fast
    def test_establishments_in_country(self) -> None:
        """Test: 'Establishments in country' pattern."""
        result = separator_lists_fixing("تأسيسات", "in", "establishments in")
        assert result == "تأسيسات في"

    @pytest.mark.fast
    def test_people_from_country(self) -> None:
        """Test: 'People from country' pattern."""
        result = add_in_tab("أشخاص", "people", "from")
        assert result == "أشخاص من "

    @pytest.mark.fast
    def test_events_at_location(self) -> None:
        """Test: 'Events at location' pattern."""
        result = separator_lists_fixing("أحداث", "at", "events at")
        assert result == "أحداث في"

    @pytest.mark.fast
    def test_buildings_and_structures_in(self) -> None:
        """Test: 'Buildings and structures in' pattern."""
        result = separator_lists_fixing("مبانٍ ومنشآت", "in", "buildings and structures in")
        assert result == "مبانٍ ومنشآت في"

    @pytest.mark.fast
    def test_organizations_based_in(self) -> None:
        """Test: 'Organizations based in' pattern."""
        result = separator_lists_fixing("منظمات", "in", "organizations based in")
        assert result == "منظمات في"


class TestDataConsistency:
    """Tests to ensure data consistency and invariants."""

    @pytest.mark.fast
    def test_idempotency_separator_fixing(self) -> None:
        """Test that applying separator_lists_fixing twice gives same result."""
        label = "منشآت عسكرية"
        result1 = separator_lists_fixing(label, "in", "military installations in")
        result2 = separator_lists_fixing(result1, "in", "military installations in")
        assert result1 == result2

    @pytest.mark.fast
    def test_idempotency_add_in_tab(self) -> None:
        """Test that applying add_in_tab twice gives same result."""
        label = "رياضيون"
        result1 = add_in_tab(label, "athletes", "from")
        result2 = add_in_tab(result1, "athletes", "from")
        assert result1 == result2

    @pytest.mark.fast
    def test_no_double_spaces(self) -> None:
        """Test that functions don't create double spaces."""
        result1 = separator_lists_fixing("منشآت", "in", "installations in")
        assert "  " not in result1

        result2 = add_in_tab("رياضيون", "athletes", "from")
        # Single space before من is expected
        assert result2.count("  ") == 0 or result2.endswith("من ")

    @pytest.mark.fast
    def test_preserves_arabic_text_integrity(self) -> None:
        """Test that Arabic text is not corrupted."""
        original = "المنشآت العسكرية الحديثة"
        result = separator_lists_fixing(original, "in", "modern military installations in")
        # Should preserve the original text and only add في
        assert result.startswith(original)
        assert len(result) >= len(original)


if __name__ == "__main__":
    pytest.main([__file__, "-v", "-m", "fast"])



"""
Tests
"""

import pytest

from ArWikiCats import resolve_label_ar

get_squad_title_data = {
    "1970 fiba women's basketball world cup": "كأس العالم لكرة السلة للسيدات 1970",
    "1970 fiba world championship for women": "بطولة كأس العالم لكرة السلة للسيدات 1970",
    "1970 fiba women's world championship": "بطولة كأس العالم لكرة السلة للسيدات 1970",
    "1970 afc asian cup": "كأس آسيا 1970",
    "1970 afc women's asian cup": "كأس الأمم الآسيوية لكرة القدم للسيدات 1970",
    "1970 afc women's championship": "بطولة آسيا للسيدات 1970",
    "1970 africa cup-of-nations": "كأس الأمم الإفريقية 1970",
    "1970 african cup of nations": "كأس الأمم الإفريقية 1970",
    "1970 african women's championship": "كأس أمم إفريقيا لكرة القدم للسيدات 1970",
    "1970 basketball olympic": "كرة سلة أولمبية 1970",
    "1970 concacaf championships": "بطولات الكونكاكاف 1970",
    "1970 concacaf gold cup": "كأس الكونكاكاف الذهبية 1970",
    "1970 concacaf women's championship": "بطولة أمريكا الشمالية للسيدات 1970",
    "1970 copa américa femenina": "كوبا أمريكا فمنينا 1970",
    "1970 copa américa": "كوبا أمريكا 1970",
    "1970 cricket world cup": "كأس العالم للكريكت 1970",
    "1970 european men's handball championship": "بطولة أوروبا لكرة اليد للرجال 1970",
    "1970 european women's handball championship": "بطولة أوروبا لكرة اليد للسيدات 1970",
    "1970 fiba asia championship": "بطولة أمم آسيا لكرة السلة 1970",
    "1970 fiba asia cup": "كأس أمم آسيا لكرة السلة 1970",
    "1970 fiba basketball world cup": "كأس العالم لكرة السلة 1970",
    "1970 fiba world championship": "بطولة كأس العالم لكرة السلة 1970",
    "1970 fiba world cup": "كأس العالم لكرة السلة 1970",
    "1970 fifa confederations cup": "كأس القارات 1970",
    "1970 fifa women's world cup": "كأس العالم لكرة القدم للسيدات 1970",
    "1970 men's hockey world cup": "كأس العالم للهوكي للرجال 1970",
    "1970 oceania cup": "كأس أوقيانوسيا 1970",
    "1970 ofc nations cup": "كأس أوقيانوسيا للأمم 1970",
    "1970 pan american games": "دورة الألعاب الأمريكية 1970",
    "1970 rugby league world cup": "كأس العالم لدوري الرجبي 1970",
    "1970 rugby world cup": "كأس العالم للرجبي 1970",
    "1970 south american championship (argentina)": "كأس أمريكا الجنوبية (الأرجنتين) 1970",
    "1970 south american championship (ecuador)": "كأس أمريكا الجنوبية (الإكوادور) 1970",
    "1970 south american championship": "بطولة أمريكا الجنوبية 1970",
    "1970 summer olympics basketball": "كرة السلة في الألعاب الأولمبية الصيفية 1970",
    "1970 summer olympics field hockey": "هوكي الميدان في الألعاب الأولمبية الصيفية 1970",
    "1970 summer olympics football": "كرة القدم في الألعاب الأولمبية الصيفية 1970",
    "1970 summer olympics handball": "كرة اليد في الألعاب الأولمبية الصيفية 1970",
    "1970 summer olympics rugby sevens": "سباعيات الرجبي في الألعاب الأولمبية الصيفية 1970",
    "1970 summer olympics volleyball": "كرة الطائرة في الألعاب الأولمبية الصيفية 1970",
    "1970 summer olympics water polo": "كرة الماء في الألعاب الأولمبية الصيفية 1970",
    "1970 summer olympics": "الألعاب الأولمبية الصيفية 1970",
    "1970 women's cricket world cup": "كأس العالم للكريكت للسيدات 1970",
    "1970 women's field hockey world cup": "كأس العالم لهوكي الميدان للسيدات 1970",
    "1970 women's hockey world cup": "كأس العالم للهوكي للسيدات 1970",
    "1970 women's rugby world cup": "كأس العالم للرجبي للسيدات 1970",
    "1970 world men's handball championship": "بطولة العالم لكرة اليد للرجال 1970",
    "1970 world women's handball championship": "بطولة العالم لكرة اليد للسيدات 1970",
}


@pytest.mark.parametrize("category, expected_key", get_squad_title_data.items(), ids=get_squad_title_data.keys())
@pytest.mark.fast
def test_get_squad_title_data(category: str, expected_key: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected_key


def test_get_squad_title() -> None:
    # Test with a basic input
    result = resolve_label_ar("test squad")
    assert isinstance(result, str)

    result_empty = resolve_label_ar("")
    assert isinstance(result_empty, str)

    # Test with various inputs
    result_various = resolve_label_ar("football team")
    assert isinstance(result_various, str)



#
import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats import resolve_arabic_category_label

religions_data = {
    "Albanian Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة ألبان",
    "Algerian Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة جزائريون",
    "American Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة أمريكيون",
    "Arab Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة عرب",
    "Australian Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة أستراليون",
    "Bangladeshi Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة بنغلاديشيون",
    "Bosnian Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة بوسنيون",
    "Egyptian Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة مصريون",
    "English Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة إنجليز",
    "Filipino Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة فلبينيون",
    "Indian Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة هنود",
    "Indonesian Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة إندونيسيون",
    "Iraqi Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة عراقيون",
    "Mauritanian Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة موريتانيون",
    "Moroccan Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة مغاربة",
    "Pakistani Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة باكستانيون",
    "Palestinian Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة فلسطينيون",
    "Persian Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة فرس",
    "Saudi Arabian Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة سعوديون",
    "Somalian Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة صوماليون",
    "Syrian Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة سوريون",
    "Turkish Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة أتراك",
    "Yemeni Sunni Muslim scholars of Islam": "تصنيف:باحثون عن الإسلام مسلمون سنة يمنيون",
    "13th-century indian muslims": "تصنيف:هنود مسلمون في القرن 13",
    "14th-century indian muslims": "تصنيف:هنود مسلمون في القرن 14",
    "15th-century indian muslims": "تصنيف:هنود مسلمون في القرن 15",
    "16th-century indian muslims": "تصنيف:هنود مسلمون في القرن 16",
    "17th-century indian muslims": "تصنيف:هنود مسلمون في القرن 17",
    "18th-century indian muslims": "تصنيف:هنود مسلمون في القرن 18",
    "19th-century indian muslims": "تصنيف:هنود مسلمون في القرن 19",
    "20th-century indian muslims": "تصنيف:هنود مسلمون في القرن 20",
    "21st-century indian muslims": "تصنيف:هنود مسلمون في القرن 21",
    "afghan muslims": "تصنيف:أفغان مسلمون",
    "african muslims": "تصنيف:أفارقة مسلمون",
    "african-american muslims": "تصنيف:أمريكيون أفارقة مسلمون",
    "albanian muslims": "تصنيف:ألبان مسلمون",
    "algerian muslims": "تصنيف:جزائريون مسلمون",
    "american muslims": "تصنيف:أمريكيون مسلمون",
    "arab muslims": "تصنيف:عرب مسلمون",
    "argentine muslims": "تصنيف:أرجنتينيون مسلمون",
    "armenian muslims": "تصنيف:أرمن مسلمون",
    "asian muslims": "تصنيف:آسيويون مسلمون",
    "australian muslims": "تصنيف:أستراليون مسلمون",
    "austrian muslims": "تصنيف:نمساويون مسلمون",
    "azerbaijani muslims": "تصنيف:أذربيجانيون مسلمون",
    "bahraini muslims": "تصنيف:بحرينيون مسلمون",
    "bangladeshi muslims": "تصنيف:بنغلاديشيون مسلمون",
    "basque muslims": "تصنيف:باسكيون مسلمون",
    "belarusian muslims": "تصنيف:بيلاروسيون مسلمون",
    "belgian muslims": "تصنيف:بلجيكيون مسلمون",
    "bengali muslims": "تصنيف:بنغاليون مسلمون",
    "beninese muslims": "تصنيف:بنينيون مسلمون",
    "bosnia and herzegovina muslims": "تصنيف:بوسنيون مسلمون",
    "bosnian muslim collaborators with nazi germany": "تصنيف:بوسنيون مسلمون متعاونون مع ألمانيا النازية",
    "bosnian muslims from ottoman empire": "تصنيف:بوسنيون مسلمون من الدولة العثمانية",
    "bosnian muslims": "تصنيف:بوسنيون مسلمون",
    "brazilian muslims": "تصنيف:برازيليون مسلمون",
    "british muslims": "تصنيف:بريطانيون مسلمون",
    "bruneian muslims": "تصنيف:برونيون مسلمون",
    "bulgarian muslims": "تصنيف:بلغاريون مسلمون",
    "burkinabe muslims": "تصنيف:بوركينابيون مسلمون",
    "burmese muslims": "تصنيف:بورميون مسلمون",
    "burundian muslims": "تصنيف:بورونديون مسلمون",
    "byzantine muslims": "تصنيف:بيزنطيون مسلمون",
    "cambodian muslims": "تصنيف:كمبوديون مسلمون",
    "cameroonian muslims": "تصنيف:كاميرونيون مسلمون",
    "canadian muslims": "تصنيف:كنديون مسلمون",
    "central african republic muslims": "تصنيف:أفارقة أوسطيون مسلمون",
    "central asian muslims": "تصنيف:آسيويون أوسطيون مسلمون",
    "chadian muslims": "تصنيف:تشاديون مسلمون",
    "chinese muslims": "تصنيف:صينيون مسلمون",
    "colombian muslims": "تصنيف:كولومبيون مسلمون",
    "comorian muslims": "تصنيف:قمريون مسلمون",
    "croatian muslims": "تصنيف:كروات مسلمون",
    "cuban muslims": "تصنيف:كوبيون مسلمون",
    "cypriot muslims": "تصنيف:قبرصيون مسلمون",
    "czech muslims": "تصنيف:تشيكيون مسلمون",
    "danish muslims": "تصنيف:دنماركيون مسلمون",
    "democratic republic of congo muslims": "تصنيف:كونغويون ديمقراطيون مسلمون",
    "dutch muslims": "تصنيف:هولنديون مسلمون",
    "east timorese muslims": "تصنيف:تيموريون شرقيون مسلمون",
    "egyptian muslims": "تصنيف:مصريون مسلمون",
    "emirati muslims": "تصنيف:إماراتيون مسلمون",
    "english muslims": "تصنيف:إنجليز مسلمون",
    "eritrean muslims": "تصنيف:إريتريون مسلمون",
    "ethiopian muslims": "تصنيف:إثيوبيون مسلمون",
    "european muslims": "تصنيف:أوروبيون مسلمون",
    "fijian muslims": "تصنيف:فيجيون مسلمون",
    "filipino muslims": "تصنيف:فلبينيون مسلمون",
    "finnish muslims": "تصنيف:فنلنديون مسلمون",
    "french muslims": "تصنيف:فرنسيون مسلمون",
    "gabonese muslims": "تصنيف:غابونيون مسلمون",
    "gambian muslims": "تصنيف:غامبيون مسلمون",
    "georgian muslims": "تصنيف:جورجيون مسلمون",
    "german muslims": "تصنيف:ألمان مسلمون",
    "ghanaian muslims": "تصنيف:غانيون مسلمون",
    "greek muslims": "تصنيف:يونانيون مسلمون",
    "guinean muslims": "تصنيف:غينيون مسلمون",
    "gujarati muslims": "تصنيف:غجراتيون مسلمون",
    "guyanese muslims": "تصنيف:غيانيون مسلمون",
    "haitian muslims": "تصنيف:هايتيون مسلمون",
    "hong kong muslims": "تصنيف:هونغ كونغيون مسلمون",
    "hungarian muslims": "تصنيف:مجريون مسلمون",
    "icelandic muslims": "تصنيف:آيسلنديون مسلمون",
    "indian muslims by century": "تصنيف:هنود مسلمون حسب القرن",
    "indian muslims": "تصنيف:هنود مسلمون",
    "indonesian muslims": "تصنيف:إندونيسيون مسلمون",
    "iranian muslims": "تصنيف:إيرانيون مسلمون",
    "iraqi muslims": "تصنيف:عراقيون مسلمون",
    "irish muslims": "تصنيف:أيرلنديون مسلمون",
    "israeli muslims": "تصنيف:إسرائيليون مسلمون",
    "italian muslims": "تصنيف:إيطاليون مسلمون",
    "ivorian muslims": "تصنيف:إيفواريون مسلمون",
    "jamaican muslims": "تصنيف:جامايكيون مسلمون",
    "japanese muslims": "تصنيف:يابانيون مسلمون",
    "jordanian muslims": "تصنيف:أردنيون مسلمون",
    "kashmiri muslims": "تصنيف:كشميريون مسلمون",
    "kazakhstani muslims": "تصنيف:كازاخستانيون مسلمون",
    "kenyan muslims": "تصنيف:كينيون مسلمون",
    "korean muslims": "تصنيف:كوريون مسلمون",
    "kosovan muslims": "تصنيف:كوسوفيون مسلمون",
    "kurdish muslims": "تصنيف:أكراد مسلمون",
    "kuwaiti muslims": "تصنيف:كويتيون مسلمون",
    "kyrgyzstani muslims": "تصنيف:قيرغيزستانيون مسلمون",
    "latvian muslims": "تصنيف:لاتفيون مسلمون",
    "lebanese muslims": "تصنيف:لبنانيون مسلمون",
    "liberian muslims": "تصنيف:ليبيريون مسلمون",
    "libyan muslims": "تصنيف:ليبيون مسلمون",
    "macedonian muslims": "تصنيف:مقدونيون مسلمون",
    "malagasy muslims": "تصنيف:مدغشقريون مسلمون",
    "malawian muslims": "تصنيف:ملاويون مسلمون",
    "malaysian muslims": "تصنيف:ماليزيون مسلمون",
    "maldivian muslims": "تصنيف:مالديفيون مسلمون",
    "malian muslims": "تصنيف:ماليون مسلمون",
    "mass murder of sri lankan muslims": "تصنيف:قتل جماعي في سريلانكيون مسلمون",
    "massacres of bengali muslims": "تصنيف:مذابح في بنغاليون مسلمون",
    "mauritanian muslims": "تصنيف:موريتانيون مسلمون",
    "mauritian muslims": "تصنيف:موريشيوسيون مسلمون",
    "mexican muslims": "تصنيف:مكسيكيون مسلمون",
    "middle eastern muslims": "تصنيف:شرقيون أوسطيون مسلمون",
    "moldovan muslims": "تصنيف:مولدوفيون مسلمون",
    "mongolian muslims": "تصنيف:منغوليون مسلمون",
    "montenegrin muslims": "تصنيف:مونتينيغريون مسلمون",
    "moroccan muslims": "تصنيف:مغاربة مسلمون",
    "mozambican muslims": "تصنيف:موزمبيقيون مسلمون",
    "namibian muslims": "تصنيف:ناميبيون مسلمون",
    "nepalese muslims": "تصنيف:نيباليون مسلمون",
    "nigerian muslims": "تصنيف:نيجيريون مسلمون",
    "nigerien muslims": "تصنيف:نيجريون مسلمون",
    "north american muslims": "تصنيف:أمريكيون شماليون مسلمون",
    "norwegian muslims": "تصنيف:نرويجيون مسلمون",
    "oceanian muslims": "تصنيف:أوقيانوسيون مسلمون",
    "omani muslims": "تصنيف:عمانيون مسلمون",
    "pakistani muslims": "تصنيف:باكستانيون مسلمون",
    "palestinian muslims": "تصنيف:فلسطينيون مسلمون",
    "panamanian muslims": "تصنيف:بنميون مسلمون",
    "persecution of bengali muslims": "تصنيف:اضطهاد بنغاليون مسلمون",
    "persecution of ottoman muslims": "تصنيف:اضطهاد عثمانيون مسلمون",
    "peruvian muslims": "تصنيف:بيرويون مسلمون",
    "polish muslims": "تصنيف:بولنديون مسلمون",
    "portuguese muslims": "تصنيف:برتغاليون مسلمون",
    "puerto rican muslims": "تصنيف:بورتوريكيون مسلمون",
    "qatari muslims": "تصنيف:قطريون مسلمون",
    "romanian muslims": "تصنيف:رومان مسلمون",
    "russian muslims": "تصنيف:روس مسلمون",
    "rwandan muslims": "تصنيف:روانديون مسلمون",
    "salvadoran muslims": "تصنيف:سلفادوريون مسلمون",
    "saudi arabian muslims": "تصنيف:سعوديون مسلمون",
    "scottish muslims": "تصنيف:إسكتلنديون مسلمون",
    "senegalese muslims": "تصنيف:سنغاليون مسلمون",
    "serbian muslims": "تصنيف:صرب مسلمون",
    "sierra leonean muslims": "تصنيف:سيراليونيون مسلمون",
    "singaporean muslims": "تصنيف:سنغافوريون مسلمون",
    "somalian muslims": "تصنيف:صوماليون مسلمون",
    "south american muslims": "تصنيف:أمريكيون جنوبيون مسلمون",
    "south korean muslims": "تصنيف:كوريون جنوبيون مسلمون",
    "south sudanese muslims": "تصنيف:سودانيون جنوبيون مسلمون",
    "soviet muslims": "تصنيف:سوفيت مسلمون",
    "spanish muslims": "تصنيف:إسبان مسلمون",
    "sri lankan muslims": "تصنيف:سريلانكيون مسلمون",
    "sudanese muslims": "تصنيف:سودانيون مسلمون",
    "surinamese muslims": "تصنيف:سوريناميون مسلمون",
    "swedish muslims": "تصنيف:سويديون مسلمون",
    "swiss muslims": "تصنيف:سويسريون مسلمون",
    "syrian muslims": "تصنيف:سوريون مسلمون",
    "taiwanese muslims": "تصنيف:تايوانيون مسلمون",
    "tajikistani muslims": "تصنيف:طاجيك مسلمون",
    "tamil muslims": "تصنيف:تاميليون مسلمون",
    "tanzanian muslims": "تصنيف:تنزانيون مسلمون",
    "thai muslims": "تصنيف:تايلنديون مسلمون",
    "tongan muslims": "تصنيف:تونغانيون مسلمون",
    "trinidad and tobago muslims": "تصنيف:ترنيداديون مسلمون",
    "tunisian muslims": "تصنيف:تونسيون مسلمون",
    "turkish cypriot muslims": "تصنيف:قبرصيون شماليون مسلمون",
    "turkish muslims": "تصنيف:أتراك مسلمون",
    "turkmenistan muslims": "تصنيف:تركمانيون مسلمون",
    "ugandan muslims": "تصنيف:أوغنديون مسلمون",
    "ukrainian muslims": "تصنيف:أوكرانيون مسلمون",
    "uruguayan muslims": "تصنيف:أوروغويانيون مسلمون",
    "uzbekistani muslims": "تصنيف:أوزبكستانيون مسلمون",
    "venezuelan muslims": "تصنيف:فنزويليون مسلمون",
    "vietnamese muslims": "تصنيف:فيتناميون مسلمون",
    "welsh muslims": "تصنيف:ويلزيون مسلمون",
    "yemeni muslims": "تصنيف:يمنيون مسلمون",
    "yugoslav muslims": "تصنيف:يوغسلافيون مسلمون",
    "zambian muslims": "تصنيف:زامبيون مسلمون",
    "zimbabwean muslims": "تصنيف:زيمبابويون مسلمون",
    "anti-muslim sentiment": "تصنيف:مشاعر معادية للمسلمون",
}

data2 = {
    "muslim league (2050–2050)": "تصنيف:الدوري المسلم (2050–2050)",
    "timelines of muslim history": "تصنيف:التسلسل الزمني تاريخ مسلم",
    "muslim families": "تصنيف:عائلات مسلمة",
    "muslim festivals": "تصنيف:مهرجانات مسلمة",
    "muslim league (1947–1958)": "تصنيف:الدوري المسلم (1947–1958)",
    "muslim members of united states house of representatives": "تصنيف:مسلمون أعضاء في مجلس النواب الأمريكي",
    "muslim nationalism in south asia": "تصنيف:قومية مسلمة في جنوب آسيا",
    "bengali muslim surnames": "تصنيف:ألقاب مسلمة بنغالية",
    "muslim rebellions": "تصنيف:تمردات مسلمة",
}


@pytest.mark.parametrize("category, expected", religions_data.items(), ids=religions_data.keys())
def test_muslims(category: str, expected: str) -> None:
    assert resolve_arabic_category_label(category) == expected


to_test = [
    ("test_muslims_1", religions_data),
    # ("test_muslims_2", data2),
]


@pytest.mark.parametrize("name,data", to_test)
@pytest.mark.dump
def test_dump_it(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_arabic_category_label)
    dump_diff(diff_result, name)

    # add_result = {x: v for x, v in data.items() if x in diff_result and "" == diff_result.get(x)}
    # dump_diff(add_result, f"{name}_add")
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



import functools
import re

REGEX_MENS = re.compile(r"\b(men)\b", re.I)
REGEX_THE = re.compile(r"\b(the)\b", re.I)


@functools.lru_cache(maxsize=10000)
def fix_keys(category: str) -> str:
    category = category.lower().replace("category:", "")
    category = category.replace("'", "")

    replacements = {
        "expatriates": "expatriate",
        "canadian football": "canadian-football",
    }

    for old, new in replacements.items():
        category = category.replace(old, new)

    category = REGEX_THE.sub("", category)
    category = REGEX_MENS.sub("mens", category)
    return category.strip()



#
import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats import resolve_arabic_category_label

data0 = {}

data_1 = {
    "Category:1690s missing person cases": "تصنيف:قضايا أشخاص مفقودين في عقد 1690",
    "Category:1620s missing person cases": "تصنيف:قضايا أشخاص مفقودين في عقد 1620",
    "Category:1580s missing person cases": "تصنيف:قضايا أشخاص مفقودين في عقد 1580",
    "Category:1540s missing person cases": "تصنيف:قضايا أشخاص مفقودين في عقد 1540",
    "Category:1550s missing person cases": "تصنيف:قضايا أشخاص مفقودين في عقد 1550",
    "Category:1510s missing person cases": "تصنيف:قضايا أشخاص مفقودين في عقد 1510",
    "Category:1520s missing person cases": "تصنيف:قضايا أشخاص مفقودين في عقد 1520",
    "Category:1500s missing person cases": "تصنيف:قضايا أشخاص مفقودين في عقد 1500",
    "Category:1590s missing person cases": "تصنيف:قضايا أشخاص مفقودين في عقد 1590",
    "Category:1610s missing person cases": "تصنيف:قضايا أشخاص مفقودين في عقد 1610",
    "Category:1630s missing person cases": "تصنيف:قضايا أشخاص مفقودين في عقد 1630",
    "Category:1670s missing person cases": "تصنيف:قضايا أشخاص مفقودين في عقد 1670",
    "Category:1570s missing person cases": "تصنيف:قضايا أشخاص مفقودين في عقد 1570",
    "Category:1680s missing person cases": "تصنيف:قضايا أشخاص مفقودين في عقد 1680",
    "Category:1460s missing person cases": "تصنيف:قضايا أشخاص مفقودين في عقد 1460",
    "Category:1410s missing person cases": "تصنيف:قضايا أشخاص مفقودين في عقد 1410",
    "Category:Film people from Northern Ireland": "تصنيف:أعلام أفلام من أيرلندا الشمالية",
    "Category:20th-century architects from Northern Ireland": "تصنيف:معماريون من أيرلندا الشمالية في القرن 20",
    "Category:20th-century engineers from Northern Ireland": "تصنيف:مهندسون من أيرلندا الشمالية في القرن 20",
    "Category:20th-century musicians by instrument from Northern Ireland": "تصنيف:موسيقيون في القرن 20 حسب الآلة من أيرلندا الشمالية",
    "Category:21st-century engineers from Northern Ireland": "تصنيف:مهندسون من أيرلندا الشمالية في القرن 21",
    "Category:21st-century male artists from Northern Ireland": "تصنيف:فنانون ذكور من أيرلندا الشمالية في القرن 21",
    "Category:21st-century medical doctors from Northern Ireland": "تصنيف:أطباء من أيرلندا الشمالية في القرن 21",
    "Category:21st-century musicians by instrument from Northern Ireland": "تصنيف:موسيقيون في القرن 21 حسب الآلة من أيرلندا الشمالية",
    "Category:21st-century painters from Northern Ireland": "تصنيف:رسامون من أيرلندا الشمالية في القرن 21",
    "Category:21st-century women educators from Northern Ireland": "تصنيف:معلمات من أيرلندا الشمالية في القرن 21",
    "Category:21st-century women medical doctors from Northern Ireland": "تصنيف:طبيبات من أيرلندا الشمالية في القرن 21",
    "Category:Accountants from Northern Ireland": "تصنيف:محاسبون من أيرلندا الشمالية",
    "Category:Anarchists from Northern Ireland": "تصنيف:لاسلطويون من أيرلندا الشمالية",
    "Category:Archers from Northern Ireland": "تصنيف:نبالون من أيرلندا الشمالية",
    "Category:Art historians from Northern Ireland": "تصنيف:مؤرخو الفن من أيرلندا الشمالية",
    "Category:Audio engineers from Northern Ireland": "تصنيف:مهندسو صوت من أيرلندا الشمالية",
    "Category:Badminton players from Northern Ireland": "تصنيف:لاعبو تنس ريشة من أيرلندا الشمالية",
    "Category:Basketball players from Northern Ireland": "تصنيف:لاعبو كرة سلة من أيرلندا الشمالية",
    "Category:Bassoonists from Northern Ireland": "تصنيف:عازفو باسون من أيرلندا الشمالية",
    "Category:Canoeists from Northern Ireland": "تصنيف:متسابقو قوارب الكانوي من أيرلندا الشمالية",
    "Category:Cardiologists from Northern Ireland": "تصنيف:أطباء قلب من أيرلندا الشمالية",
    "Category:Choreographers from Northern Ireland": "تصنيف:مصممو رقص من أيرلندا الشمالية",
    "Category:Classical scholars from Northern Ireland": "تصنيف:باحثون كلاسيكيون من أيرلندا الشمالية",
    "Category:Comedy writers from Northern Ireland": "تصنيف:كتاب كوميديا من أيرلندا الشمالية",
    "Category:Dance music groups from Northern Ireland": "تصنيف:فرق موسيقى الرقص من أيرلندا الشمالية",
    "Category:Documentary filmmakers from Northern Ireland": "تصنيف:صانعو أفلام وثائقية من أيرلندا الشمالية",
    "Category:Editorial cartoonists from Northern Ireland": "تصنيف:محررون كارتونيون من أيرلندا الشمالية",
    "Category:Editors from Northern Ireland": "تصنيف:محررون من أيرلندا الشمالية",
    "Category:Electronic music groups from Northern Ireland": "تصنيف:فرق موسيقى إلكترونية من أيرلندا الشمالية",
    "Category:Epidemiologists from Northern Ireland": "تصنيف:علماء وبائيات من أيرلندا الشمالية",
    "Category:Explorers from Northern Ireland": "تصنيف:مستكشفون من أيرلندا الشمالية",
    "Category:Female archers from Northern Ireland": "تصنيف:نبالات من أيرلندا الشمالية",
    "Category:Female badminton players from Northern Ireland": "تصنيف:لاعبات تنس ريشة من أيرلندا الشمالية",
    "Category:Female equestrians from Northern Ireland": "تصنيف:فارسات خيول من أيرلندا الشمالية",
    "Category:Female golfers from Northern Ireland": "تصنيف:لاعبات غولف من أيرلندا الشمالية",
    "Category:Female judoka from Northern Ireland": "تصنيف:لاعبات جودو من أيرلندا الشمالية",
    "Category:Female martial artists from Northern Ireland": "تصنيف:ممارسات فنون قتالية من أيرلندا الشمالية",
    "Category:Female sport shooters from Northern Ireland": "تصنيف:لاعبات رماية من أيرلندا الشمالية",
    "Category:Female swimmers from Northern Ireland": "تصنيف:سباحات من أيرلندا الشمالية",
    "Category:Feminists from Northern Ireland": "تصنيف:نسويون من أيرلندا الشمالية",
    "Category:Fiddlers from Northern Ireland": "تصنيف:عازفو كمان من أيرلندا الشمالية",
    "Category:Film critics from Northern Ireland": "تصنيف:نقاد أفلام من أيرلندا الشمالية",
    "Category:Film editors from Northern Ireland": "تصنيف:محررو أفلام من أيرلندا الشمالية",
    "Category:Films scored by composers from Northern Ireland": "تصنيف:أفلام سجلها ملحنون من أيرلندا الشمالية",
    "Category:Folk music groups from Northern Ireland": "تصنيف:فرق موسيقى تقليدية من أيرلندا الشمالية",
    "Category:Gaelic footballers from Northern Ireland by county": "تصنيف:لاعبو كرة قدم غيلية من أيرلندا الشمالية حسب المقاطعة",
    "Category:Gaelic footballers from Northern Ireland": "تصنيف:لاعبو كرة قدم غيلية من أيرلندا الشمالية",
    "Category:Gamblers from Northern Ireland": "تصنيف:مقامرون من أيرلندا الشمالية",
    "Category:Geologists from Northern Ireland": "تصنيف:جيولوجيون من أيرلندا الشمالية",
    "Category:Horticulturists from Northern Ireland": "تصنيف:بستانيون من أيرلندا الشمالية",
    "Category:Humorists from Northern Ireland": "تصنيف:فكاهيون من أيرلندا الشمالية",
    "Category:Installation artists from Northern Ireland": "تصنيف:فنانون تنصيبيون من أيرلندا الشمالية",
    "Category:Jazz musicians from Northern Ireland": "تصنيف:موسيقيو جاز من أيرلندا الشمالية",
    "Category:Legal scholars from Northern Ireland": "تصنيف:أساتذة قانون من أيرلندا الشمالية",
    "Category:Lists of politicians from Northern Ireland": "تصنيف:قوائم سياسيون من أيرلندا الشمالية",
    "Category:Local politicians in Northern Ireland": "تصنيف:سياسيون محليون في أيرلندا الشمالية",
    "Category:Lyricists from Northern Ireland": "تصنيف:شعراء غنائيون من أيرلندا الشمالية",
    "Category:Male chefs from Northern Ireland": "تصنيف:طباخون ذكور من أيرلندا الشمالية",
    "Category:Male fiddlers from Northern Ireland": "تصنيف:عازفو كمان ذكور من أيرلندا الشمالية",
    "Category:Male sailors (sport) from Northern Ireland": "تصنيف:بحارة رياضيون ذكور من أيرلندا الشمالية",
    "Category:Male sport shooters from Northern Ireland": "تصنيف:لاعبو رماية ذكور من أيرلندا الشمالية",
    "Category:Marxists from Northern Ireland": "تصنيف:ماركسيون من أيرلندا الشمالية",
    "Category:Meteorologists from Northern Ireland": "تصنيف:خبراء أرصاد جوية من أيرلندا الشمالية",
    "Category:Music critics from Northern Ireland": "تصنيف:نقاد موسيقى من أيرلندا الشمالية",
    "Category:Music journalists from Northern Ireland": "تصنيف:صحفيون موسيقيون من أيرلندا الشمالية",
    "Category:Music video directors from Northern Ireland": "تصنيف:مخرجو فيديوهات موسيقية من أيرلندا الشمالية",
    "Category:Musicologists from Northern Ireland": "تصنيف:علماء موسيقى من أيرلندا الشمالية",
    "Category:Nature writers from Northern Ireland": "تصنيف:كتاب طبيعة من أيرلندا الشمالية",
    "Category:Netball players from Northern Ireland": "تصنيف:لاعبو كرة شبكة من أيرلندا الشمالية",
    "Category:Neurologists from Northern Ireland": "تصنيف:أطباء أعصاب من أيرلندا الشمالية",
    "Category:Neuroscientists from Northern Ireland": "تصنيف:علماء أعصاب من أيرلندا الشمالية",
    "Category:Newspaper editors from Northern Ireland": "تصنيف:محررو صحف من أيرلندا الشمالية",
    "Category:Newspaper people by newspaper in Northern Ireland": "تصنيف:شخصيات صحف حسب الصحيفة في أيرلندا الشمالية",
    "Category:Newspaper people from Northern Ireland": "تصنيف:شخصيات صحف من أيرلندا الشمالية",
    "Category:Nurses from Northern Ireland": "تصنيف:ممرضون من أيرلندا الشمالية",
    "Category:Organists from Northern Ireland": "تصنيف:عازفو أورغن من أيرلندا الشمالية",
    "Category:Ornithologists from Northern Ireland": "تصنيف:علماء طيور من أيرلندا الشمالية",
    "Category:Poker players from Northern Ireland": "تصنيف:لاعبو بوكر من أيرلندا الشمالية",
    "Category:Political scientists from Northern Ireland": "تصنيف:علماء سياسة من أيرلندا الشمالية",
    "Category:Political writers from Northern Ireland": "تصنيف:كتاب سياسة من أيرلندا الشمالية",
    "Category:Pop music groups from Northern Ireland": "تصنيف:فرق موسيقى بوب من أيرلندا الشمالية",
    "Category:Radio actresses from Northern Ireland": "تصنيف:ممثلات راديو من أيرلندا الشمالية",
    "Category:Radio producers from Northern Ireland": "تصنيف:منتجو راديو من أيرلندا الشمالية",
    "Category:Radiologists from Northern Ireland": "تصنيف:أخصائيو أشعة من أيرلندا الشمالية",
    "Category:Sailors (sport) from Northern Ireland": "تصنيف:بحارة رياضيون من أيرلندا الشمالية",
    "Category:Satirists from Northern Ireland": "تصنيف:كتاب هجائيون من أيرلندا الشمالية",
    "Category:Silent film actors from Northern Ireland": "تصنيف:ممثلو أفلام صامتة من أيرلندا الشمالية",
    "Category:Sport shooters from Northern Ireland": "تصنيف:لاعبو رماية من أيرلندا الشمالية",
    "Category:Sports broadcasters from Northern Ireland": "تصنيف:مذيعون رياضيون من أيرلندا الشمالية",
    "Category:Sportspeople from Northern Ireland in doping cases": "تصنيف:رياضيون من أيرلندا الشمالية في قضايا منشطات",
    "Category:Statisticians from Northern Ireland": "تصنيف:إحصائيون من أيرلندا الشمالية",
    "Category:Surrealist artists from Northern Ireland": "تصنيف:فنانون سرياليون من أيرلندا الشمالية",
    "Category:Table tennis players from Northern Ireland": "تصنيف:لاعبو كرة طاولة من أيرلندا الشمالية",
    "Category:Television chefs from Northern Ireland": "تصنيف:طباخو تلفاز من أيرلندا الشمالية",
    "Category:Television talk show hosts from Northern Ireland": "تصنيف:مقدمو برامج تلفزيونية حوارية من أيرلندا الشمالية",
    "Category:Video artists from Northern Ireland": "تصنيف:فنانو فيديو من أيرلندا الشمالية",
    "Category:Watercolourists from Northern Ireland": "تصنيف:رسامون مائيون من أيرلندا الشمالية",
    "Category:Women academics from Northern Ireland": "تصنيف:أكاديميات من أيرلندا الشمالية",
    "Category:Women boxers from Northern Ireland": "تصنيف:ملاكمات من أيرلندا الشمالية",
    "Category:Women chefs from Northern Ireland": "تصنيف:طباخات من أيرلندا الشمالية",
    "Category:Women choreographers from Northern Ireland": "تصنيف:مصممات رقص من أيرلندا الشمالية",
    "Category:Women civil servants from Northern Ireland": "تصنيف:موظفات خدمة مدنية من أيرلندا الشمالية",
    "Category:Women comedians from Northern Ireland": "تصنيف:كوميديات من أيرلندا الشمالية",
    "Category:Women judges from Northern Ireland": "تصنيف:قاضيات من أيرلندا الشمالية",
    "Category:Women mathematicians from Northern Ireland": "تصنيف:عالمات رياضيات من أيرلندا الشمالية",
    "Category:Women mayors of places in Northern Ireland": "تصنيف:رئيسات بلديات في أيرلندا الشمالية",
    "Category:Women members of the Parliament of Northern Ireland": "تصنيف:عضوات برلمان أيرلندا الشمالية",
    "Category:Women photographers from Northern Ireland": "تصنيف:مصورات من أيرلندا الشمالية",
    "Category:Women television personalities from Northern Ireland": "تصنيف:شخصيات تلفزيون من أيرلندا الشمالية",
    "Category:Women travel writers from Northern Ireland": "تصنيف:كاتبات سفر من أيرلندا الشمالية",
}

data_2 = {
    "Category:20th-century MEPs for Northern Ireland": "",
    "Category:20th-century Presbyterian ministers from Northern Ireland": "",
    "Category:21st-century MEPs for Northern Ireland": "",
    "Category:21st-century Presbyterian ministers from Northern Ireland": "",
    "Category:21st-century women MEPs for Northern Ireland": "",
    "Category:Advocates general for Northern Ireland": "",
    "Category:All-female bands from Northern Ireland": "",
    "Category:Alliance Party of Northern Ireland politicians": "",
    "Category:Anaesthetists from Northern Ireland": "",
    "Category:Antiquarians from Northern Ireland": "",
    "Category:Artistic directors from Northern Ireland": "",
    "Category:Banjoists from Northern Ireland": "",
    "Category:Barristers from Northern Ireland": "",
    "Category:Blues musical groups from Northern Ireland": "",
    "Category:Bouzouki players from Northern Ireland": "",
    "Category:Boxing promoters from Northern Ireland": "",
    "Category:Ceremonial officers in Northern Ireland": "",
    "Category:Chief Medical Officers for Northern Ireland": "",
    "Category:Children's entertainers from Northern Ireland": "",
    "Category:Children's television presenters from Northern Ireland": "",
    "Category:Civil servants in the Northern Ireland Office": "",
    "Category:Cookbook writers from Northern Ireland": "",
    "Category:Councillors in Northern Ireland": "",
    "Category:Deputy lieutenants in Northern Ireland": "",
    "Category:Drag queens from Northern Ireland": "",
    "Category:Experimental musical groups from Northern Ireland": "",
    "Category:Far-right politicians from Northern Ireland": "",
    "Category:Film score composers from Northern Ireland": "",
    "Category:Folk-song collectors from Northern Ireland": "",
    "Category:Game show hosts from Northern Ireland": "",
    "Category:Gay actors from Northern Ireland": "",
    "Category:Gay men from Northern Ireland by occupation": "",
    "Category:Gay musicians from Northern Ireland": "",
    "Category:Gay writers from Northern Ireland": "",
    "Category:General practitioners from Northern Ireland": "",
    "Category:Green Party Northern Ireland politicians": "",
    "Category:Hammered dulcimer players from Northern Ireland": "",
    "Category:Harpists from Northern Ireland": "",
    "Category:Heads of the Northern Ireland Civil Service": "",
    "Category:Health professionals from Northern Ireland by county": "",
    "Category:Health professionals from Northern Ireland": "",
    "Category:High Court judges of Northern Ireland": "",
    "Category:High shrievalties in Northern Ireland": "",
    "Category:Hurlers from Northern Ireland by county": "",
    "Category:Hurlers from Northern Ireland": "",
    "Category:Hymnwriters from Northern Ireland": "",
    "Category:Independent politicians in Northern Ireland": "",
    "Category:Ladies' Gaelic footballers from Northern Ireland": "",
    "Category:Leaders of the Alliance Party of Northern Ireland": "",
    "Category:LGBTQ comedians from Northern Ireland": "",
    "Category:LGBTQ dramatists and playwrights from Northern Ireland": "",
    "Category:LGBTQ models from Northern Ireland": "",
    "Category:Local government officers in Northern Ireland": "",
    "Category:Lord chief justices of Northern Ireland": "",
    "Category:Lord-lieutenants in Northern Ireland": "",
    "Category:Lords Justice of Appeal of Northern Ireland": "",
    "Category:Male harpists from Northern Ireland": "",
    "Category:Male uilleann pipers from Northern Ireland": "",
    "Category:Marxist writers from Northern Ireland": "",
    "Category:Mass media people from Northern Ireland by county": "",
    "Category:Media personalities from Northern Ireland": "",
    "Category:Members of the Northern Ireland Civil Service": "",
    "Category:Members of the Northern Ireland Constitutional Convention": "",
    "Category:Members of the Northern Ireland Forum": "",
    "Category:Members of the Privy Council of Northern Ireland": "",
    "Category:Methodist ministers from Northern Ireland": "",
    "Category:Musical duos from Northern Ireland": "",
    "Category:Musical quartets from Northern Ireland": "",
    "Category:Musical quintets from Northern Ireland": "",
    "Category:Musical theatre composers from Northern Ireland": "",
    "Category:Musical trios from Northern Ireland": "",
    "Category:National Democratic Party (Northern Ireland) politicians": "",
    "Category:Noise musical groups from Northern Ireland": "",
    "Category:Northern Ireland (UK) MP stubs": "",
    "Category:Northern Ireland Assembly member stubs": "",
    "Category:Northern Ireland Government ministers": "",
    "Category:Northern Ireland justices of the peace": "",
    "Category:Northern Ireland King's Counsel": "",
    "Category:Northern Ireland Labour Party politicians": "",
    "Category:Northern Ireland MLAs 1998–2003": "",
    "Category:Northern Ireland politician stubs": "",
    "Category:Northern Ireland prison officers": "",
    "Category:Northern Ireland Unionist Party politicians": "",
    "Category:Northern Ireland Women's Coalition politicians": "",
    "Category:Oboists from Northern Ireland": "",
    "Category:Ombudsmen in Northern Ireland": "",
    "Category:Orchestras from Northern Ireland": "",
    "Category:Palaeontologists from Northern Ireland": "",
    "Category:Parliament of Northern Ireland member stubs": "",
    "Category:Permanent under-secretaries of state for Northern Ireland": "",
    "Category:Players of English billiards from Northern Ireland": "",
    "Category:Pool players from Northern Ireland": "",
    "Category:Presbyterian ministers from Northern Ireland": "",
    "Category:Psychologists from Northern Ireland": "",
    "Category:Punk rock musicians from Northern Ireland": "",
    "Category:Radio DJs from Northern Ireland": "",
    "Category:Roboticists from Northern Ireland": "",
    "Category:Shakespearean actors from Northern Ireland": "",
    "Category:Songs by songwriters from Northern Ireland": "",
    "Category:Sportspeople from Northern Ireland stubs": "",
    "Category:Strength athletes from Northern Ireland": "",
    "Category:TikTokers from Northern Ireland": "",
    "Category:Tin whistle players from Northern Ireland": "",
    "Category:Uilleann pipers from Northern Ireland": "",
    "Category:Wikipedia categories named after actors from Northern Ireland": "",
    "Category:Wikipedia categories named after musical groups from Northern Ireland": "",
    "Category:Wikipedia categories named after musicians from Northern Ireland": "",
    "Category:Women councillors in Northern Ireland": "",
    "Category:Women harpists from Northern Ireland": "",
    "Category:Women MEPs for Northern Ireland": "",
    "Category:Women political office-holders in Northern Ireland": "",
}

to_test = [
    ("test_northern_ireland_1", data_1),
    # ("test_northern_ireland_2", data_2),
]


@pytest.mark.parametrize("category, expected", data_1.items(), ids=data_1.keys())
@pytest.mark.fast
def test_northern_ireland_1(category: str, expected: str) -> None:
    label = resolve_arabic_category_label(category)
    assert label == expected


@pytest.mark.parametrize("name,data", to_test)
@pytest.mark.dump
def test_dump_all(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_arabic_category_label)

    dump_diff(diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



"""Canonical public API for gendered job datasets.

This package aggregates the typed datasets exposed across the refactored job
modules and publishes a stable import surface for downstream callers.  Importers
can rely on :mod:`translations.jobs` to retrieve commonly used mappings without
pulling in individual module internals.
"""

from __future__ import annotations



#
import pytest
from load_one_data import dump_diff, dump_diff_text, one_dump_test

from ArWikiCats import resolve_arabic_category_label, resolve_label_ar

data0 = {}

data_1 = {
    # FINE
    "14th-century writers from Yemen": "كتاب من اليمن في القرن 14",
    "14th-century football players from Yemen": "لاعبو كرة قدم من اليمن في القرن 14",
    "wheelchair basketball players from Yemen": "لاعبو كرة سلة على كراسي متحركة من اليمن",
    "1400s BC wheelchair basketball players from Yemen": "لاعبو كرة سلة على كراسي متحركة من اليمن في عقد 1400 ق م",
}

data_2 = {
    # NEED FIXING
    "2020 international footballers from Yemen": "لاعبو كرة قدم دوليون من اليمن في 2020",
    "14th-century writers from the democratic republic of the Congo": "كتاب من جمهورية الكونغو الديمقراطية في القرن 14",
    "14th-century writers from Crown of Aragon": "كتاب من تاج أرغون في القرن 14",
    "14th-century writers from the Crown of Aragon": "كتاب من تاج أرغون في القرن 14",
}

to_test = [
    ("test_fix_in_min_0", data0),
    ("test_fix_in_min_1", data_1),
    ("test_fix_in_min_2", data_2),
]


@pytest.mark.parametrize("category, expected", data_1.items(), ids=data_1.keys())
@pytest.mark.fast
def test_fix_in_min_1(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", data_2.items(), ids=data_2.keys())
@pytest.mark.skip2
def test_fix_in_min_2(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


@pytest.mark.parametrize("name,data", to_test)
@pytest.mark.dump
def test_dump_all(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_label_ar)
    dump_diff(diff_result, name)
    # dump_diff_text(expected, diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"
